# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AggregationData,
    Alert,
    AlertAlertType,
    AlertPagePublic,
    AlertPublic,
    AlertPublicAlertType,
    AlertTrigger,
    AlertTriggerConfig,
    AlertTriggerConfigPublic,
    AlertTriggerConfigPublicType,
    AlertTriggerConfigType,
    AlertTriggerConfigWrite,
    AlertTriggerConfigWriteType,
    AlertTriggerEventType,
    AlertTriggerPublic,
    AlertTriggerPublicEventType,
    AlertTriggerWrite,
    AlertTriggerWriteEventType,
    AlertWrite,
    AlertWriteAlertType,
    AnnotationQueue,
    AnnotationQueueBatch,
    AnnotationQueueItemIds,
    AnnotationQueuePagePublic,
    AnnotationQueuePublic,
    AnnotationQueuePublicScope,
    AnnotationQueueReviewer,
    AnnotationQueueReviewerPublic,
    AnnotationQueueScope,
    AnnotationQueueWrite,
    AnnotationQueueWriteScope,
    AssistantMessage,
    AssistantMessageRole,
    Attachment,
    AttachmentPage,
    AudioUrl,
    AudioUrlPublic,
    AudioUrlWrite,
    AuthDetailsHolder,
    AutomationRuleEvaluator,
    AutomationRuleEvaluatorAction,
    AutomationRuleEvaluatorLlmAsJudge,
    AutomationRuleEvaluatorLlmAsJudgePublic,
    AutomationRuleEvaluatorLlmAsJudgeWrite,
    AutomationRuleEvaluatorObjectObjectPublic,
    AutomationRuleEvaluatorObjectObjectPublicAction,
    AutomationRuleEvaluatorObjectObjectPublic_LlmAsJudge,
    AutomationRuleEvaluatorObjectObjectPublic_SpanLlmAsJudge,
    AutomationRuleEvaluatorObjectObjectPublic_SpanUserDefinedMetricPython,
    AutomationRuleEvaluatorObjectObjectPublic_TraceThreadLlmAsJudge,
    AutomationRuleEvaluatorObjectObjectPublic_TraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorObjectObjectPublic_UserDefinedMetricPython,
    AutomationRuleEvaluatorPagePublic,
    AutomationRuleEvaluatorPublic,
    AutomationRuleEvaluatorPublicAction,
    AutomationRuleEvaluatorPublic_LlmAsJudge,
    AutomationRuleEvaluatorPublic_SpanLlmAsJudge,
    AutomationRuleEvaluatorPublic_SpanUserDefinedMetricPython,
    AutomationRuleEvaluatorPublic_TraceThreadLlmAsJudge,
    AutomationRuleEvaluatorPublic_TraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorPublic_UserDefinedMetricPython,
    AutomationRuleEvaluatorSpanLlmAsJudge,
    AutomationRuleEvaluatorSpanLlmAsJudgePublic,
    AutomationRuleEvaluatorSpanLlmAsJudgeWrite,
    AutomationRuleEvaluatorSpanUserDefinedMetricPython,
    AutomationRuleEvaluatorSpanUserDefinedMetricPythonPublic,
    AutomationRuleEvaluatorSpanUserDefinedMetricPythonWrite,
    AutomationRuleEvaluatorTraceThreadLlmAsJudge,
    AutomationRuleEvaluatorTraceThreadLlmAsJudgePublic,
    AutomationRuleEvaluatorTraceThreadLlmAsJudgeWrite,
    AutomationRuleEvaluatorTraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorTraceThreadUserDefinedMetricPythonPublic,
    AutomationRuleEvaluatorTraceThreadUserDefinedMetricPythonWrite,
    AutomationRuleEvaluatorUpdate,
    AutomationRuleEvaluatorUpdateAction,
    AutomationRuleEvaluatorUpdateLlmAsJudge,
    AutomationRuleEvaluatorUpdateSpanLlmAsJudge,
    AutomationRuleEvaluatorUpdateSpanUserDefinedMetricPython,
    AutomationRuleEvaluatorUpdateTraceThreadLlmAsJudge,
    AutomationRuleEvaluatorUpdateTraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorUpdateUserDefinedMetricPython,
    AutomationRuleEvaluatorUpdate_LlmAsJudge,
    AutomationRuleEvaluatorUpdate_SpanLlmAsJudge,
    AutomationRuleEvaluatorUpdate_SpanUserDefinedMetricPython,
    AutomationRuleEvaluatorUpdate_TraceThreadLlmAsJudge,
    AutomationRuleEvaluatorUpdate_TraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorUpdate_UserDefinedMetricPython,
    AutomationRuleEvaluatorUserDefinedMetricPython,
    AutomationRuleEvaluatorUserDefinedMetricPythonPublic,
    AutomationRuleEvaluatorUserDefinedMetricPythonWrite,
    AutomationRuleEvaluatorWrite,
    AutomationRuleEvaluatorWriteAction,
    AutomationRuleEvaluatorWrite_LlmAsJudge,
    AutomationRuleEvaluatorWrite_SpanLlmAsJudge,
    AutomationRuleEvaluatorWrite_SpanUserDefinedMetricPython,
    AutomationRuleEvaluatorWrite_TraceThreadLlmAsJudge,
    AutomationRuleEvaluatorWrite_TraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorWrite_UserDefinedMetricPython,
    AutomationRuleEvaluator_LlmAsJudge,
    AutomationRuleEvaluator_SpanLlmAsJudge,
    AutomationRuleEvaluator_SpanUserDefinedMetricPython,
    AutomationRuleEvaluator_TraceThreadLlmAsJudge,
    AutomationRuleEvaluator_TraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluator_UserDefinedMetricPython,
    AvgValueStatPublic,
    BatchDelete,
    BatchDeleteByProject,
    BiInformation,
    BiInformationResponse,
    BooleanFeedbackDefinition,
    BooleanFeedbackDefinitionCreate,
    BooleanFeedbackDefinitionPublic,
    BooleanFeedbackDefinitionUpdate,
    BooleanFeedbackDetail,
    BooleanFeedbackDetailCreate,
    BooleanFeedbackDetailPublic,
    BooleanFeedbackDetailUpdate,
    BreakdownConfigPublic,
    BreakdownConfigPublicField,
    CategoricalFeedbackDefinition,
    CategoricalFeedbackDefinitionCreate,
    CategoricalFeedbackDefinitionPublic,
    CategoricalFeedbackDefinitionUpdate,
    CategoricalFeedbackDetail,
    CategoricalFeedbackDetailCreate,
    CategoricalFeedbackDetailPublic,
    CategoricalFeedbackDetailUpdate,
    ChatCompletionChoice,
    ChatCompletionResponse,
    Check,
    CheckName,
    CheckPublic,
    CheckPublicName,
    CheckPublicResult,
    CheckResult,
    ChunkedOutputJsonNode,
    ChunkedOutputJsonNodePublic,
    ChunkedOutputJsonNodePublicType,
    ChunkedOutputJsonNodeType,
    Column,
    ColumnCompare,
    ColumnCompareTypesItem,
    ColumnPublic,
    ColumnPublicTypesItem,
    ColumnTypesItem,
    Comment,
    CommentCompare,
    CommentPublic,
    CompleteMultipartUploadRequest,
    CompleteMultipartUploadRequestEntityType,
    CompletionTokensDetails,
    CountValueStatPublic,
    DashboardPagePublic,
    DashboardPublic,
    DataPointDouble,
    DataPointNumberPublic,
    Dataset,
    DatasetExpansion,
    DatasetExpansionResponse,
    DatasetExportJobPublic,
    DatasetExportJobPublicStatus,
    DatasetItem,
    DatasetItemBatch,
    DatasetItemChangesPublic,
    DatasetItemCompare,
    DatasetItemCompareSource,
    DatasetItemFilter,
    DatasetItemFilterOperator,
    DatasetItemPageCompare,
    DatasetItemPagePublic,
    DatasetItemPublic,
    DatasetItemPublicSource,
    DatasetItemSource,
    DatasetItemUpdate,
    DatasetItemWrite,
    DatasetItemWriteSource,
    DatasetPagePublic,
    DatasetPublic,
    DatasetPublicStatus,
    DatasetPublicVisibility,
    DatasetStatus,
    DatasetVersionDiff,
    DatasetVersionDiffStats,
    DatasetVersionPagePublic,
    DatasetVersionPublic,
    DatasetVersionSummary,
    DatasetVersionSummaryPublic,
    DatasetVisibility,
    DeleteAttachmentsRequest,
    DeleteAttachmentsRequestEntityType,
    DeleteFeedbackScore,
    DeleteIdsHolder,
    Delta,
    ErrorCountWithDeviation,
    ErrorCountWithDeviationDetailed,
    ErrorInfo,
    ErrorInfoExperimentItemBulkWriteView,
    ErrorInfoPublic,
    ErrorInfoWrite,
    ErrorMessage,
    ErrorMessageDetail,
    ErrorMessageDetailed,
    ErrorMessagePublic,
    Experiment,
    ExperimentGroupAggregationsResponse,
    ExperimentGroupResponse,
    ExperimentItem,
    ExperimentItemBulkRecord,
    ExperimentItemBulkRecordExperimentItemBulkWriteView,
    ExperimentItemBulkUpload,
    ExperimentItemCompare,
    ExperimentItemCompareTraceVisibilityMode,
    ExperimentItemPublic,
    ExperimentItemPublicTraceVisibilityMode,
    ExperimentItemReference,
    ExperimentItemReferencePublic,
    ExperimentItemTraceVisibilityMode,
    ExperimentPagePublic,
    ExperimentPublic,
    ExperimentPublicStatus,
    ExperimentPublicType,
    ExperimentScore,
    ExperimentScorePublic,
    ExperimentScoreWrite,
    ExperimentStatus,
    ExperimentType,
    ExperimentUpdate,
    ExperimentUpdateStatus,
    ExperimentUpdateType,
    ExportTraceServiceRequest,
    Feedback,
    FeedbackCreate,
    FeedbackCreate_Boolean,
    FeedbackCreate_Categorical,
    FeedbackCreate_Numerical,
    FeedbackDefinitionPagePublic,
    FeedbackObjectPublic,
    FeedbackObjectPublic_Boolean,
    FeedbackObjectPublic_Categorical,
    FeedbackObjectPublic_Numerical,
    FeedbackPublic,
    FeedbackPublic_Boolean,
    FeedbackPublic_Categorical,
    FeedbackPublic_Numerical,
    FeedbackScore,
    FeedbackScoreAverage,
    FeedbackScoreAverageDetailed,
    FeedbackScoreAveragePublic,
    FeedbackScoreBatch,
    FeedbackScoreBatchItem,
    FeedbackScoreBatchItemSource,
    FeedbackScoreBatchItemThread,
    FeedbackScoreBatchItemThreadSource,
    FeedbackScoreCompare,
    FeedbackScoreCompareSource,
    FeedbackScoreExperimentItemBulkWriteView,
    FeedbackScoreExperimentItemBulkWriteViewSource,
    FeedbackScoreNames,
    FeedbackScorePublic,
    FeedbackScorePublicSource,
    FeedbackScoreSource,
    FeedbackUpdate,
    FeedbackUpdate_Boolean,
    FeedbackUpdate_Categorical,
    FeedbackUpdate_Numerical,
    Feedback_Boolean,
    Feedback_Categorical,
    Feedback_Numerical,
    Function,
    FunctionCall,
    GroupContent,
    GroupContentWithAggregations,
    GroupDetail,
    GroupDetails,
    Guardrail,
    GuardrailBatch,
    GuardrailName,
    GuardrailResult,
    GuardrailWrite,
    GuardrailWriteName,
    GuardrailWriteResult,
    GuardrailsValidation,
    GuardrailsValidationPublic,
    IdsHolder,
    ImageUrl,
    ImageUrlPublic,
    ImageUrlWrite,
    JsonListString,
    JsonListStringCompare,
    JsonListStringExperimentItemBulkWriteView,
    JsonListStringPublic,
    JsonListStringWrite,
    JsonNode,
    JsonNodeDetail,
    JsonNodePublic,
    JsonNodeWrite,
    JsonSchema,
    LlmAsJudgeCode,
    LlmAsJudgeCodePublic,
    LlmAsJudgeCodeWrite,
    LlmAsJudgeMessage,
    LlmAsJudgeMessageContent,
    LlmAsJudgeMessageContentPublic,
    LlmAsJudgeMessageContentWrite,
    LlmAsJudgeMessagePublic,
    LlmAsJudgeMessagePublicRole,
    LlmAsJudgeMessageRole,
    LlmAsJudgeMessageWrite,
    LlmAsJudgeMessageWriteRole,
    LlmAsJudgeModelParameters,
    LlmAsJudgeModelParametersPublic,
    LlmAsJudgeModelParametersWrite,
    LlmAsJudgeOutputSchema,
    LlmAsJudgeOutputSchemaPublic,
    LlmAsJudgeOutputSchemaPublicType,
    LlmAsJudgeOutputSchemaType,
    LlmAsJudgeOutputSchemaWrite,
    LlmAsJudgeOutputSchemaWriteType,
    LogItem,
    LogItemLevel,
    LogPage,
    ManualEvaluationRequest,
    ManualEvaluationRequestEntityType,
    ManualEvaluationResponse,
    Message,
    MultipartUploadPart,
    NumericalFeedbackDefinition,
    NumericalFeedbackDefinitionCreate,
    NumericalFeedbackDefinitionPublic,
    NumericalFeedbackDefinitionUpdate,
    NumericalFeedbackDetail,
    NumericalFeedbackDetailCreate,
    NumericalFeedbackDetailPublic,
    NumericalFeedbackDetailUpdate,
    OllamaConnectionTestResponse,
    OllamaInstanceBaseUrlRequest,
    OllamaModel,
    Optimization,
    OptimizationPagePublic,
    OptimizationPublic,
    OptimizationPublicStatus,
    OptimizationStatus,
    OptimizationStudioConfig,
    OptimizationStudioConfigPublic,
    OptimizationStudioConfigWrite,
    OptimizationStudioLog,
    OptimizationWrite,
    OptimizationWriteStatus,
    PageColumns,
    PercentageValueStatPublic,
    PercentageValues,
    PercentageValuesDetailed,
    PercentageValuesPublic,
    Project,
    ProjectDetailed,
    ProjectDetailedVisibility,
    ProjectMetricResponsePublic,
    ProjectMetricResponsePublicInterval,
    ProjectMetricResponsePublicMetricType,
    ProjectPagePublic,
    ProjectPublic,
    ProjectPublicVisibility,
    ProjectReference,
    ProjectReferencePublic,
    ProjectStatItemObjectPublic,
    ProjectStatItemObjectPublic_Avg,
    ProjectStatItemObjectPublic_Count,
    ProjectStatItemObjectPublic_Percentage,
    ProjectStatsPublic,
    ProjectStatsSummary,
    ProjectStatsSummaryItem,
    ProjectVisibility,
    Prompt,
    PromptDetail,
    PromptDetailTemplateStructure,
    PromptPagePublic,
    PromptPublic,
    PromptPublicTemplateStructure,
    PromptTemplateStructure,
    PromptTokensDetails,
    PromptType,
    PromptVersion,
    PromptVersionDetail,
    PromptVersionDetailTemplateStructure,
    PromptVersionDetailType,
    PromptVersionLink,
    PromptVersionLinkPublic,
    PromptVersionLinkWrite,
    PromptVersionPagePublic,
    PromptVersionPublic,
    PromptVersionPublicTemplateStructure,
    PromptVersionPublicType,
    PromptVersionTemplateStructure,
    PromptVersionType,
    PromptVersionUpdate,
    ProviderApiKey,
    ProviderApiKeyPagePublic,
    ProviderApiKeyProvider,
    ProviderApiKeyPublic,
    ProviderApiKeyPublicProvider,
    ResponseFormat,
    ResponseFormatType,
    Result,
    ResultsNumberPublic,
    ScoreName,
    ServiceTogglesConfig,
    Span,
    SpanBatch,
    SpanEnrichmentOptions,
    SpanExperimentItemBulkWriteView,
    SpanExperimentItemBulkWriteViewType,
    SpanFilter,
    SpanFilterOperator,
    SpanFilterPublic,
    SpanFilterPublicOperator,
    SpanFilterWrite,
    SpanFilterWriteOperator,
    SpanLlmAsJudgeCode,
    SpanLlmAsJudgeCodePublic,
    SpanLlmAsJudgeCodeWrite,
    SpanPagePublic,
    SpanPublic,
    SpanPublicType,
    SpanType,
    SpanUpdate,
    SpanUpdateType,
    SpanUserDefinedMetricPythonCode,
    SpanUserDefinedMetricPythonCodePublic,
    SpanUserDefinedMetricPythonCodeWrite,
    SpanWrite,
    SpanWriteType,
    SpansCountResponse,
    StartMultipartUploadResponse,
    StreamOptions,
    StudioEvaluation,
    StudioEvaluationPublic,
    StudioEvaluationWrite,
    StudioLlmModel,
    StudioLlmModelPublic,
    StudioLlmModelWrite,
    StudioMessage,
    StudioMessagePublic,
    StudioMessageWrite,
    StudioMetric,
    StudioMetricPublic,
    StudioMetricWrite,
    StudioOptimizer,
    StudioOptimizerPublic,
    StudioOptimizerWrite,
    StudioPrompt,
    StudioPromptPublic,
    StudioPromptWrite,
    TokenUsageNames,
    Tool,
    ToolCall,
    ToolCallType,
    ToolType,
    Trace,
    TraceBatch,
    TraceCountResponse,
    TraceEnrichmentOptions,
    TraceExperimentItemBulkWriteView,
    TraceFilter,
    TraceFilterOperator,
    TraceFilterPublic,
    TraceFilterPublicOperator,
    TraceFilterWrite,
    TraceFilterWriteOperator,
    TracePagePublic,
    TracePublic,
    TracePublicVisibilityMode,
    TraceThread,
    TraceThreadFilter,
    TraceThreadFilterOperator,
    TraceThreadFilterPublic,
    TraceThreadFilterPublicOperator,
    TraceThreadFilterWrite,
    TraceThreadFilterWriteOperator,
    TraceThreadIdentifier,
    TraceThreadLlmAsJudgeCode,
    TraceThreadLlmAsJudgeCodePublic,
    TraceThreadLlmAsJudgeCodeWrite,
    TraceThreadPage,
    TraceThreadStatus,
    TraceThreadUpdate,
    TraceThreadUserDefinedMetricPythonCode,
    TraceThreadUserDefinedMetricPythonCodePublic,
    TraceThreadUserDefinedMetricPythonCodeWrite,
    TraceUpdate,
    TraceVisibilityMode,
    TraceWrite,
    Usage,
    UserDefinedMetricPythonCode,
    UserDefinedMetricPythonCodePublic,
    UserDefinedMetricPythonCodeWrite,
    ValueEntry,
    ValueEntryCompare,
    ValueEntryCompareSource,
    ValueEntryExperimentItemBulkWriteView,
    ValueEntryExperimentItemBulkWriteViewSource,
    ValueEntryPublic,
    ValueEntryPublicSource,
    ValueEntrySource,
    VideoUrl,
    VideoUrlPublic,
    VideoUrlWrite,
    Webhook,
    WebhookExamples,
    WebhookPublic,
    WebhookTestResult,
    WebhookTestResultStatus,
    WebhookWrite,
    WelcomeWizardTracking,
    WorkspaceConfiguration,
    WorkspaceMetricRequest,
    WorkspaceMetricResponse,
    WorkspaceMetricsSummaryRequest,
    WorkspaceMetricsSummaryResponse,
    WorkspaceNameHolder,
    WorkspaceSpansCount,
    WorkspaceTraceCount,
)
from .errors import (
    BadGatewayError,
    BadRequestError,
    ConflictError,
    ForbiddenError,
    InternalServerError,
    NotFoundError,
    NotImplementedError,
    ServiceUnavailableError,
    UnauthorizedError,
    UnprocessableEntityError,
)
from . import (
    alerts,
    annotation_queues,
    attachments,
    automation_rule_evaluators,
    chat_completions,
    check,
    dashboards,
    datasets,
    experiments,
    feedback_definitions,
    guardrails,
    llm_provider_key,
    manual_evaluation,
    ollama,
    open_telemetry_ingestion,
    optimizations,
    projects,
    prompts,
    redirect,
    service_toggles,
    spans,
    system_usage,
    traces,
    welcome_wizard,
    workspaces,
)
from .alerts import GetWebhookExamplesRequestAlertType
from .attachments import (
    AttachmentListRequestEntityType,
    DownloadAttachmentRequestEntityType,
    StartMultipartUploadRequestEntityType,
    UploadAttachmentRequestEntityType,
)
from .client import AsyncOpikApi, OpikApi
from .datasets import DatasetUpdateVisibility, DatasetWriteVisibility
from .environment import OpikApiEnvironment
from .experiments import ExperimentWriteStatus, ExperimentWriteType
from .feedback_definitions import FindFeedbackDefinitionsRequestType
from .llm_provider_key import ProviderApiKeyWriteProvider
from .optimizations import OptimizationUpdateStatus
from .projects import (
    ProjectMetricRequestPublicInterval,
    ProjectMetricRequestPublicMetricType,
    ProjectUpdateVisibility,
    ProjectWriteVisibility,
)
from .prompts import CreatePromptVersionDetailTemplateStructure, PromptWriteTemplateStructure, PromptWriteType
from .spans import (
    FindFeedbackScoreNames1RequestType,
    GetSpanStatsRequestType,
    GetSpansByProjectRequestType,
    SpanSearchStreamRequestPublicType,
)

__all__ = [
    "AggregationData",
    "Alert",
    "AlertAlertType",
    "AlertPagePublic",
    "AlertPublic",
    "AlertPublicAlertType",
    "AlertTrigger",
    "AlertTriggerConfig",
    "AlertTriggerConfigPublic",
    "AlertTriggerConfigPublicType",
    "AlertTriggerConfigType",
    "AlertTriggerConfigWrite",
    "AlertTriggerConfigWriteType",
    "AlertTriggerEventType",
    "AlertTriggerPublic",
    "AlertTriggerPublicEventType",
    "AlertTriggerWrite",
    "AlertTriggerWriteEventType",
    "AlertWrite",
    "AlertWriteAlertType",
    "AnnotationQueue",
    "AnnotationQueueBatch",
    "AnnotationQueueItemIds",
    "AnnotationQueuePagePublic",
    "AnnotationQueuePublic",
    "AnnotationQueuePublicScope",
    "AnnotationQueueReviewer",
    "AnnotationQueueReviewerPublic",
    "AnnotationQueueScope",
    "AnnotationQueueWrite",
    "AnnotationQueueWriteScope",
    "AssistantMessage",
    "AssistantMessageRole",
    "AsyncOpikApi",
    "Attachment",
    "AttachmentListRequestEntityType",
    "AttachmentPage",
    "AudioUrl",
    "AudioUrlPublic",
    "AudioUrlWrite",
    "AuthDetailsHolder",
    "AutomationRuleEvaluator",
    "AutomationRuleEvaluatorAction",
    "AutomationRuleEvaluatorLlmAsJudge",
    "AutomationRuleEvaluatorLlmAsJudgePublic",
    "AutomationRuleEvaluatorLlmAsJudgeWrite",
    "AutomationRuleEvaluatorObjectObjectPublic",
    "AutomationRuleEvaluatorObjectObjectPublicAction",
    "AutomationRuleEvaluatorObjectObjectPublic_LlmAsJudge",
    "AutomationRuleEvaluatorObjectObjectPublic_SpanLlmAsJudge",
    "AutomationRuleEvaluatorObjectObjectPublic_SpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorObjectObjectPublic_TraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorObjectObjectPublic_TraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorObjectObjectPublic_UserDefinedMetricPython",
    "AutomationRuleEvaluatorPagePublic",
    "AutomationRuleEvaluatorPublic",
    "AutomationRuleEvaluatorPublicAction",
    "AutomationRuleEvaluatorPublic_LlmAsJudge",
    "AutomationRuleEvaluatorPublic_SpanLlmAsJudge",
    "AutomationRuleEvaluatorPublic_SpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorPublic_TraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorPublic_TraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorPublic_UserDefinedMetricPython",
    "AutomationRuleEvaluatorSpanLlmAsJudge",
    "AutomationRuleEvaluatorSpanLlmAsJudgePublic",
    "AutomationRuleEvaluatorSpanLlmAsJudgeWrite",
    "AutomationRuleEvaluatorSpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorSpanUserDefinedMetricPythonPublic",
    "AutomationRuleEvaluatorSpanUserDefinedMetricPythonWrite",
    "AutomationRuleEvaluatorTraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorTraceThreadLlmAsJudgePublic",
    "AutomationRuleEvaluatorTraceThreadLlmAsJudgeWrite",
    "AutomationRuleEvaluatorTraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorTraceThreadUserDefinedMetricPythonPublic",
    "AutomationRuleEvaluatorTraceThreadUserDefinedMetricPythonWrite",
    "AutomationRuleEvaluatorUpdate",
    "AutomationRuleEvaluatorUpdateAction",
    "AutomationRuleEvaluatorUpdateLlmAsJudge",
    "AutomationRuleEvaluatorUpdateSpanLlmAsJudge",
    "AutomationRuleEvaluatorUpdateSpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorUpdateTraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorUpdateTraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorUpdateUserDefinedMetricPython",
    "AutomationRuleEvaluatorUpdate_LlmAsJudge",
    "AutomationRuleEvaluatorUpdate_SpanLlmAsJudge",
    "AutomationRuleEvaluatorUpdate_SpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorUpdate_TraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorUpdate_TraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorUpdate_UserDefinedMetricPython",
    "AutomationRuleEvaluatorUserDefinedMetricPython",
    "AutomationRuleEvaluatorUserDefinedMetricPythonPublic",
    "AutomationRuleEvaluatorUserDefinedMetricPythonWrite",
    "AutomationRuleEvaluatorWrite",
    "AutomationRuleEvaluatorWriteAction",
    "AutomationRuleEvaluatorWrite_LlmAsJudge",
    "AutomationRuleEvaluatorWrite_SpanLlmAsJudge",
    "AutomationRuleEvaluatorWrite_SpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorWrite_TraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorWrite_TraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorWrite_UserDefinedMetricPython",
    "AutomationRuleEvaluator_LlmAsJudge",
    "AutomationRuleEvaluator_SpanLlmAsJudge",
    "AutomationRuleEvaluator_SpanUserDefinedMetricPython",
    "AutomationRuleEvaluator_TraceThreadLlmAsJudge",
    "AutomationRuleEvaluator_TraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluator_UserDefinedMetricPython",
    "AvgValueStatPublic",
    "BadGatewayError",
    "BadRequestError",
    "BatchDelete",
    "BatchDeleteByProject",
    "BiInformation",
    "BiInformationResponse",
    "BooleanFeedbackDefinition",
    "BooleanFeedbackDefinitionCreate",
    "BooleanFeedbackDefinitionPublic",
    "BooleanFeedbackDefinitionUpdate",
    "BooleanFeedbackDetail",
    "BooleanFeedbackDetailCreate",
    "BooleanFeedbackDetailPublic",
    "BooleanFeedbackDetailUpdate",
    "BreakdownConfigPublic",
    "BreakdownConfigPublicField",
    "CategoricalFeedbackDefinition",
    "CategoricalFeedbackDefinitionCreate",
    "CategoricalFeedbackDefinitionPublic",
    "CategoricalFeedbackDefinitionUpdate",
    "CategoricalFeedbackDetail",
    "CategoricalFeedbackDetailCreate",
    "CategoricalFeedbackDetailPublic",
    "CategoricalFeedbackDetailUpdate",
    "ChatCompletionChoice",
    "ChatCompletionResponse",
    "Check",
    "CheckName",
    "CheckPublic",
    "CheckPublicName",
    "CheckPublicResult",
    "CheckResult",
    "ChunkedOutputJsonNode",
    "ChunkedOutputJsonNodePublic",
    "ChunkedOutputJsonNodePublicType",
    "ChunkedOutputJsonNodeType",
    "Column",
    "ColumnCompare",
    "ColumnCompareTypesItem",
    "ColumnPublic",
    "ColumnPublicTypesItem",
    "ColumnTypesItem",
    "Comment",
    "CommentCompare",
    "CommentPublic",
    "CompleteMultipartUploadRequest",
    "CompleteMultipartUploadRequestEntityType",
    "CompletionTokensDetails",
    "ConflictError",
    "CountValueStatPublic",
    "CreatePromptVersionDetailTemplateStructure",
    "DashboardPagePublic",
    "DashboardPublic",
    "DataPointDouble",
    "DataPointNumberPublic",
    "Dataset",
    "DatasetExpansion",
    "DatasetExpansionResponse",
    "DatasetExportJobPublic",
    "DatasetExportJobPublicStatus",
    "DatasetItem",
    "DatasetItemBatch",
    "DatasetItemChangesPublic",
    "DatasetItemCompare",
    "DatasetItemCompareSource",
    "DatasetItemFilter",
    "DatasetItemFilterOperator",
    "DatasetItemPageCompare",
    "DatasetItemPagePublic",
    "DatasetItemPublic",
    "DatasetItemPublicSource",
    "DatasetItemSource",
    "DatasetItemUpdate",
    "DatasetItemWrite",
    "DatasetItemWriteSource",
    "DatasetPagePublic",
    "DatasetPublic",
    "DatasetPublicStatus",
    "DatasetPublicVisibility",
    "DatasetStatus",
    "DatasetUpdateVisibility",
    "DatasetVersionDiff",
    "DatasetVersionDiffStats",
    "DatasetVersionPagePublic",
    "DatasetVersionPublic",
    "DatasetVersionSummary",
    "DatasetVersionSummaryPublic",
    "DatasetVisibility",
    "DatasetWriteVisibility",
    "DeleteAttachmentsRequest",
    "DeleteAttachmentsRequestEntityType",
    "DeleteFeedbackScore",
    "DeleteIdsHolder",
    "Delta",
    "DownloadAttachmentRequestEntityType",
    "ErrorCountWithDeviation",
    "ErrorCountWithDeviationDetailed",
    "ErrorInfo",
    "ErrorInfoExperimentItemBulkWriteView",
    "ErrorInfoPublic",
    "ErrorInfoWrite",
    "ErrorMessage",
    "ErrorMessageDetail",
    "ErrorMessageDetailed",
    "ErrorMessagePublic",
    "Experiment",
    "ExperimentGroupAggregationsResponse",
    "ExperimentGroupResponse",
    "ExperimentItem",
    "ExperimentItemBulkRecord",
    "ExperimentItemBulkRecordExperimentItemBulkWriteView",
    "ExperimentItemBulkUpload",
    "ExperimentItemCompare",
    "ExperimentItemCompareTraceVisibilityMode",
    "ExperimentItemPublic",
    "ExperimentItemPublicTraceVisibilityMode",
    "ExperimentItemReference",
    "ExperimentItemReferencePublic",
    "ExperimentItemTraceVisibilityMode",
    "ExperimentPagePublic",
    "ExperimentPublic",
    "ExperimentPublicStatus",
    "ExperimentPublicType",
    "ExperimentScore",
    "ExperimentScorePublic",
    "ExperimentScoreWrite",
    "ExperimentStatus",
    "ExperimentType",
    "ExperimentUpdate",
    "ExperimentUpdateStatus",
    "ExperimentUpdateType",
    "ExperimentWriteStatus",
    "ExperimentWriteType",
    "ExportTraceServiceRequest",
    "Feedback",
    "FeedbackCreate",
    "FeedbackCreate_Boolean",
    "FeedbackCreate_Categorical",
    "FeedbackCreate_Numerical",
    "FeedbackDefinitionPagePublic",
    "FeedbackObjectPublic",
    "FeedbackObjectPublic_Boolean",
    "FeedbackObjectPublic_Categorical",
    "FeedbackObjectPublic_Numerical",
    "FeedbackPublic",
    "FeedbackPublic_Boolean",
    "FeedbackPublic_Categorical",
    "FeedbackPublic_Numerical",
    "FeedbackScore",
    "FeedbackScoreAverage",
    "FeedbackScoreAverageDetailed",
    "FeedbackScoreAveragePublic",
    "FeedbackScoreBatch",
    "FeedbackScoreBatchItem",
    "FeedbackScoreBatchItemSource",
    "FeedbackScoreBatchItemThread",
    "FeedbackScoreBatchItemThreadSource",
    "FeedbackScoreCompare",
    "FeedbackScoreCompareSource",
    "FeedbackScoreExperimentItemBulkWriteView",
    "FeedbackScoreExperimentItemBulkWriteViewSource",
    "FeedbackScoreNames",
    "FeedbackScorePublic",
    "FeedbackScorePublicSource",
    "FeedbackScoreSource",
    "FeedbackUpdate",
    "FeedbackUpdate_Boolean",
    "FeedbackUpdate_Categorical",
    "FeedbackUpdate_Numerical",
    "Feedback_Boolean",
    "Feedback_Categorical",
    "Feedback_Numerical",
    "FindFeedbackDefinitionsRequestType",
    "FindFeedbackScoreNames1RequestType",
    "ForbiddenError",
    "Function",
    "FunctionCall",
    "GetSpanStatsRequestType",
    "GetSpansByProjectRequestType",
    "GetWebhookExamplesRequestAlertType",
    "GroupContent",
    "GroupContentWithAggregations",
    "GroupDetail",
    "GroupDetails",
    "Guardrail",
    "GuardrailBatch",
    "GuardrailName",
    "GuardrailResult",
    "GuardrailWrite",
    "GuardrailWriteName",
    "GuardrailWriteResult",
    "GuardrailsValidation",
    "GuardrailsValidationPublic",
    "IdsHolder",
    "ImageUrl",
    "ImageUrlPublic",
    "ImageUrlWrite",
    "InternalServerError",
    "JsonListString",
    "JsonListStringCompare",
    "JsonListStringExperimentItemBulkWriteView",
    "JsonListStringPublic",
    "JsonListStringWrite",
    "JsonNode",
    "JsonNodeDetail",
    "JsonNodePublic",
    "JsonNodeWrite",
    "JsonSchema",
    "LlmAsJudgeCode",
    "LlmAsJudgeCodePublic",
    "LlmAsJudgeCodeWrite",
    "LlmAsJudgeMessage",
    "LlmAsJudgeMessageContent",
    "LlmAsJudgeMessageContentPublic",
    "LlmAsJudgeMessageContentWrite",
    "LlmAsJudgeMessagePublic",
    "LlmAsJudgeMessagePublicRole",
    "LlmAsJudgeMessageRole",
    "LlmAsJudgeMessageWrite",
    "LlmAsJudgeMessageWriteRole",
    "LlmAsJudgeModelParameters",
    "LlmAsJudgeModelParametersPublic",
    "LlmAsJudgeModelParametersWrite",
    "LlmAsJudgeOutputSchema",
    "LlmAsJudgeOutputSchemaPublic",
    "LlmAsJudgeOutputSchemaPublicType",
    "LlmAsJudgeOutputSchemaType",
    "LlmAsJudgeOutputSchemaWrite",
    "LlmAsJudgeOutputSchemaWriteType",
    "LogItem",
    "LogItemLevel",
    "LogPage",
    "ManualEvaluationRequest",
    "ManualEvaluationRequestEntityType",
    "ManualEvaluationResponse",
    "Message",
    "MultipartUploadPart",
    "NotFoundError",
    "NotImplementedError",
    "NumericalFeedbackDefinition",
    "NumericalFeedbackDefinitionCreate",
    "NumericalFeedbackDefinitionPublic",
    "NumericalFeedbackDefinitionUpdate",
    "NumericalFeedbackDetail",
    "NumericalFeedbackDetailCreate",
    "NumericalFeedbackDetailPublic",
    "NumericalFeedbackDetailUpdate",
    "OllamaConnectionTestResponse",
    "OllamaInstanceBaseUrlRequest",
    "OllamaModel",
    "OpikApi",
    "OpikApiEnvironment",
    "Optimization",
    "OptimizationPagePublic",
    "OptimizationPublic",
    "OptimizationPublicStatus",
    "OptimizationStatus",
    "OptimizationStudioConfig",
    "OptimizationStudioConfigPublic",
    "OptimizationStudioConfigWrite",
    "OptimizationStudioLog",
    "OptimizationUpdateStatus",
    "OptimizationWrite",
    "OptimizationWriteStatus",
    "PageColumns",
    "PercentageValueStatPublic",
    "PercentageValues",
    "PercentageValuesDetailed",
    "PercentageValuesPublic",
    "Project",
    "ProjectDetailed",
    "ProjectDetailedVisibility",
    "ProjectMetricRequestPublicInterval",
    "ProjectMetricRequestPublicMetricType",
    "ProjectMetricResponsePublic",
    "ProjectMetricResponsePublicInterval",
    "ProjectMetricResponsePublicMetricType",
    "ProjectPagePublic",
    "ProjectPublic",
    "ProjectPublicVisibility",
    "ProjectReference",
    "ProjectReferencePublic",
    "ProjectStatItemObjectPublic",
    "ProjectStatItemObjectPublic_Avg",
    "ProjectStatItemObjectPublic_Count",
    "ProjectStatItemObjectPublic_Percentage",
    "ProjectStatsPublic",
    "ProjectStatsSummary",
    "ProjectStatsSummaryItem",
    "ProjectUpdateVisibility",
    "ProjectVisibility",
    "ProjectWriteVisibility",
    "Prompt",
    "PromptDetail",
    "PromptDetailTemplateStructure",
    "PromptPagePublic",
    "PromptPublic",
    "PromptPublicTemplateStructure",
    "PromptTemplateStructure",
    "PromptTokensDetails",
    "PromptType",
    "PromptVersion",
    "PromptVersionDetail",
    "PromptVersionDetailTemplateStructure",
    "PromptVersionDetailType",
    "PromptVersionLink",
    "PromptVersionLinkPublic",
    "PromptVersionLinkWrite",
    "PromptVersionPagePublic",
    "PromptVersionPublic",
    "PromptVersionPublicTemplateStructure",
    "PromptVersionPublicType",
    "PromptVersionTemplateStructure",
    "PromptVersionType",
    "PromptVersionUpdate",
    "PromptWriteTemplateStructure",
    "PromptWriteType",
    "ProviderApiKey",
    "ProviderApiKeyPagePublic",
    "ProviderApiKeyProvider",
    "ProviderApiKeyPublic",
    "ProviderApiKeyPublicProvider",
    "ProviderApiKeyWriteProvider",
    "ResponseFormat",
    "ResponseFormatType",
    "Result",
    "ResultsNumberPublic",
    "ScoreName",
    "ServiceTogglesConfig",
    "ServiceUnavailableError",
    "Span",
    "SpanBatch",
    "SpanEnrichmentOptions",
    "SpanExperimentItemBulkWriteView",
    "SpanExperimentItemBulkWriteViewType",
    "SpanFilter",
    "SpanFilterOperator",
    "SpanFilterPublic",
    "SpanFilterPublicOperator",
    "SpanFilterWrite",
    "SpanFilterWriteOperator",
    "SpanLlmAsJudgeCode",
    "SpanLlmAsJudgeCodePublic",
    "SpanLlmAsJudgeCodeWrite",
    "SpanPagePublic",
    "SpanPublic",
    "SpanPublicType",
    "SpanSearchStreamRequestPublicType",
    "SpanType",
    "SpanUpdate",
    "SpanUpdateType",
    "SpanUserDefinedMetricPythonCode",
    "SpanUserDefinedMetricPythonCodePublic",
    "SpanUserDefinedMetricPythonCodeWrite",
    "SpanWrite",
    "SpanWriteType",
    "SpansCountResponse",
    "StartMultipartUploadRequestEntityType",
    "StartMultipartUploadResponse",
    "StreamOptions",
    "StudioEvaluation",
    "StudioEvaluationPublic",
    "StudioEvaluationWrite",
    "StudioLlmModel",
    "StudioLlmModelPublic",
    "StudioLlmModelWrite",
    "StudioMessage",
    "StudioMessagePublic",
    "StudioMessageWrite",
    "StudioMetric",
    "StudioMetricPublic",
    "StudioMetricWrite",
    "StudioOptimizer",
    "StudioOptimizerPublic",
    "StudioOptimizerWrite",
    "StudioPrompt",
    "StudioPromptPublic",
    "StudioPromptWrite",
    "TokenUsageNames",
    "Tool",
    "ToolCall",
    "ToolCallType",
    "ToolType",
    "Trace",
    "TraceBatch",
    "TraceCountResponse",
    "TraceEnrichmentOptions",
    "TraceExperimentItemBulkWriteView",
    "TraceFilter",
    "TraceFilterOperator",
    "TraceFilterPublic",
    "TraceFilterPublicOperator",
    "TraceFilterWrite",
    "TraceFilterWriteOperator",
    "TracePagePublic",
    "TracePublic",
    "TracePublicVisibilityMode",
    "TraceThread",
    "TraceThreadFilter",
    "TraceThreadFilterOperator",
    "TraceThreadFilterPublic",
    "TraceThreadFilterPublicOperator",
    "TraceThreadFilterWrite",
    "TraceThreadFilterWriteOperator",
    "TraceThreadIdentifier",
    "TraceThreadLlmAsJudgeCode",
    "TraceThreadLlmAsJudgeCodePublic",
    "TraceThreadLlmAsJudgeCodeWrite",
    "TraceThreadPage",
    "TraceThreadStatus",
    "TraceThreadUpdate",
    "TraceThreadUserDefinedMetricPythonCode",
    "TraceThreadUserDefinedMetricPythonCodePublic",
    "TraceThreadUserDefinedMetricPythonCodeWrite",
    "TraceUpdate",
    "TraceVisibilityMode",
    "TraceWrite",
    "UnauthorizedError",
    "UnprocessableEntityError",
    "UploadAttachmentRequestEntityType",
    "Usage",
    "UserDefinedMetricPythonCode",
    "UserDefinedMetricPythonCodePublic",
    "UserDefinedMetricPythonCodeWrite",
    "ValueEntry",
    "ValueEntryCompare",
    "ValueEntryCompareSource",
    "ValueEntryExperimentItemBulkWriteView",
    "ValueEntryExperimentItemBulkWriteViewSource",
    "ValueEntryPublic",
    "ValueEntryPublicSource",
    "ValueEntrySource",
    "VideoUrl",
    "VideoUrlPublic",
    "VideoUrlWrite",
    "Webhook",
    "WebhookExamples",
    "WebhookPublic",
    "WebhookTestResult",
    "WebhookTestResultStatus",
    "WebhookWrite",
    "WelcomeWizardTracking",
    "WorkspaceConfiguration",
    "WorkspaceMetricRequest",
    "WorkspaceMetricResponse",
    "WorkspaceMetricsSummaryRequest",
    "WorkspaceMetricsSummaryResponse",
    "WorkspaceNameHolder",
    "WorkspaceSpansCount",
    "WorkspaceTraceCount",
    "alerts",
    "annotation_queues",
    "attachments",
    "automation_rule_evaluators",
    "chat_completions",
    "check",
    "dashboards",
    "datasets",
    "experiments",
    "feedback_definitions",
    "guardrails",
    "llm_provider_key",
    "manual_evaluation",
    "ollama",
    "open_telemetry_ingestion",
    "optimizations",
    "projects",
    "prompts",
    "redirect",
    "service_toggles",
    "spans",
    "system_usage",
    "traces",
    "welcome_wizard",
    "workspaces",
]



from typing import List, Dict

from opik.evaluation.metrics.conversation import types as conversation_types


def evaluate_conversation(sliding_window: conversation_types.Conversation) -> str:
    return f"""Based on the given list of message exchanges between a user and an LLM, generate a JSON object to indicate whether the LAST `user` message is showing that the user experiences confusion, annoyance, or disengagement during the conversation session given in the context of the last messages.

** Guidelines: **
- Make sure to only return in JSON format.
- The JSON must have only 2 fields: 'verdict' and 'reason'.
- The 'verdict' key should STRICTLY be either 'yes' or 'no', which states whether the last `user` message is showing that the user experienced confusion, annoyance, or disengagement during the conversation session given in the context of the last messages.
- Provide a 'reason' ONLY if the answer is 'yes'.
- You DON'T have to provide a reason if the answer is 'no'.
- You MUST USE the previous messages (if any) provided in the list of messages to make an informed judgement on user frustration.
- You MUST ONLY provide a verdict for the LAST message on the list but MUST USE context from the previous messages.
- ONLY provide a 'yes' answer if the LLM response caused the user to express COMPLETE frustration or confusion in his/her input messages.
- Vague LLM responses to vague inputs, such as greetings DOES NOT count as causes of frustration!
- You should mention LLM response instead of `assistant`, and User instead of `user`.

===== Start OF EXAMPLE ======
** Example Turns: **
[
    {{
        "role": "user",
        "content": "Hello!"
    }},
    {{
        "role": "assistant",
        "content": "Hi there!"
    }},
    {{
        "role": "user",
        "content": "Can you help me write a function to parse dates in Python?"
    }},
    {{
        "role": "assistant",
        "content": "Of course! Here's a basic example using `datetime.strptime`:\n\n```python\nfrom datetime import datetime\n\ndef parse_date(date_string):\n    return datetime.strptime(date_string, '%Y-%m-%d')\n```"
    }},
    {{
        "role": "user",
        "content": "That's not what I meant. I need it to handle multiple formats."
    }},
    {{
        "role": "assistant",
        "content": "Ah, got it. Here's an updated version that tries multiple formats:\n\n```python\nfrom datetime import datetime\n\ndef parse_date(date_string):\n    for fmt in ('%Y-%m-%d', '%d/%m/%Y', '%m-%d-%Y'):\n        try:\n            return datetime.strptime(date_string, fmt)\n        except ValueError:\n            continue\n    raise ValueError('No valid date format found')\n```"
    }},
    {{
        "role": "user",
        "content": "Why didn’t you just give me this the first time? This is basic."
    }},
    {{
        "role": "assistant",
        "content": "content": "Apologies for the oversight. I always aim to provide the most helpful answer based on the initial question."
    }},
    {{
        "role": "user",
        "content": "Well, it's kind of frustrating to keep asking for obvious things. Can you at least include error handling too next time?"
    }},
    {{
        "role": "assistant",
        "content": "You're right, I’ll be more thorough. Let me know if you'd like to expand the function with logging or timezone support as well."

    }},
    {{
        "role": "user",
        "content": "Now you're overcomplicating it. I just wanted a simple, robust function."
    }},
    {{
        "role": "assistant",
        "content": "Understood. Here's a simpler version with basic error handling included:\n\n```python\nfrom datetime import datetime\n\ndef parse_date(date_string):\n    formats = ('%Y-%m-%d', '%d/%m/%Y', '%m-%d-%Y')\n    for fmt in formats:\n        try:\n            return datetime.strptime(date_string, fmt)\n        except ValueError:\n            continue\n    return None  # Or raise an error if preferred\n```"
    }},
    {{
        "role": "user", "content": "You could’ve just started with this. Why is it so hard to get a straight answer?"
    }}
]

** Example output JSON **
{{
    "verdict": "yes",
    "reason": "The User repeatedly clarifies their intent and expresses dissatisfaction with the LLM’s initial responses, indicating a mismatch between the User’s expectations and the LLM’s output. Despite asking a clear question, the LLM provides an overly simplistic solution at first, requiring the User to iterate and request obvious improvements (e.g., support for multiple formats, error handling). The User’s tone becomes increasingly critical, with statements like “Why didn’t you just give me this the first time?” and “Why is it so hard to get a straight answer?”, signaling rising frustration due to perceived inefficiency and lack of responsiveness from the assistant."
}}
===== END OF EXAMPLE ======

** Turns: **
{sliding_window}

** JSON: **
"""


def generate_reason(score: float, frustrations: List[Dict[str, str]]) -> str:
    return f"""Below is a list of user frustrations drawn from some messages in a conversation, which you have minimal knowledge of. It is a list of strings explaining why the 'assistant' messages caused the 'user' to be frustrated.
Given the frustration score, which is a 0-1 score indicating how frustrating the OVERALL AI 'assistant' messages are in a conversation (higher the more frustrating).

** Guidelines: **
- Make sure to only return in JSON format, with the 'reason' key providing the reason.
- Always quote WHICH MESSAGE and the INFORMATION in the reason in your final reason.
- Be confident in your reasoning, as if you’re aware of the `assistant` messages from the messages in a conversation that led to the frustrations.
- You should CONCISELY summarize the user frustrations to justify the score.
- You should NOT mention concrete frustration in your reason, and make the reason sound convincing.
- You should mention LLM response instead of `assistant`, and User instead of `user`.
- You should format <frustration_score> to use 1 decimal place in the reason.

===== Start OF EXAMPLE ======
** Example frustrations: **
[
    {{
        "message_number": "1",
        "reason": "The User repeatedly clarifies their intent and expresses dissatisfaction with the LLM’s initial responses, indicating a mismatch between the User’s expectations and the LLM’s output."
    }},
    {{
        "message_number": "2",
        "reason": "Despite asking a clear question, the LLM provides an overly simplistic solution at first, requiring the User to iterate and request obvious improvements (e.g., support for multiple formats, error handling)."
    }},
    {{
        "message_number": "3",
        "reason": "The User’s tone becomes increasingly critical, with statements like “Why didn’t you just give me this the first time?” and “Why is it so hard to get a straight answer?”, signaling rising frustration due to perceived inefficiency and lack of responsiveness from the assistant."
    }}
]

** Example output JSON **
{{
    "reason": "The score is <frustration_score> because <your_reason>."
}}
===== END OF EXAMPLE ======

** Frustration Score: **
{score}

** Frustrations: **
{frustrations}

** JSON: **
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .manual_evaluation_request_entity_type import ManualEvaluationRequestEntityType


class ManualEvaluationRequest(UniversalBaseModel):
    project_id: str = pydantic.Field()
    """
    Project ID
    """

    entity_ids: typing.List[str] = pydantic.Field()
    """
    List of entity IDs (trace IDs or thread IDs) to evaluate
    """

    rule_ids: typing.List[str] = pydantic.Field()
    """
    List of automation rule IDs to apply
    """

    entity_type: ManualEvaluationRequestEntityType = pydantic.Field()
    """
    Type of entity to evaluate (trace or thread)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from __future__ import annotations

from typing import Optional, Union

from opik.evaluation.metrics.llm_judges.g_eval import metric as g_eval_metric
from opik.evaluation.models import base_model


class PromptUncertaintyJudge(g_eval_metric.GEvalPreset):
    """
    Rate how ambiguous or underspecified a prompt feels to the model.

    Args:
        model: Optional model identifier or ``OpikBaseModel`` instance.
        track: Whether to automatically track judge outputs. Defaults to ``True``.
        project_name: Optional tracking project name.
        temperature: Sampling temperature forwarded to the underlying model.

    Example:
        >>> from opik.evaluation.metrics import PromptUncertaintyJudge
        >>> judge = PromptUncertaintyJudge(model="gpt-4")
        >>> result = judge.score(output="Do the right thing in the best way possible.")  # doctest: +SKIP
        >>> result.value  # doctest: +SKIP
        0.8
    """

    def __init__(
        self,
        model: Optional[Union[str, base_model.OpikBaseModel]] = None,
        track: bool = True,
        project_name: Optional[str] = None,
        temperature: float = 0.0,
    ) -> None:
        super().__init__(
            preset="prompt_uncertainty",
            model=model,
            track=track,
            project_name=project_name,
            temperature=temperature,
            name="prompt_uncertainty_judge",
        )



"""
Patcher for LiteLLM completion functions used by CrewAI.

This module patches litellm.completion and litellm.acompletion with Opik tracking.
CrewAI v0.x uses LiteLLM internally for LLM calls.
"""

import logging
from typing import Optional

import litellm

import opik.integrations.litellm

LOGGER = logging.getLogger(__name__)


def patch_litellm_completion(project_name: Optional[str] = None) -> None:
    """
    Patches LiteLLM completion functions used by CrewAI.

    Args:
        project_name: The name of the project to associate with tracking.
    """
    litellm.completion = opik.integrations.litellm.track_completion(
        project_name=project_name
    )(litellm.completion)
    litellm.acompletion = opik.integrations.litellm.track_completion(
        project_name=project_name
    )(litellm.acompletion)



"""Language adherence metric leveraging fastText-style language identification."""

from __future__ import annotations

from typing import Any, Callable, Optional, Tuple

from opik.exceptions import MetricComputationError
from opik.evaluation.metrics.base_metric import BaseMetric
from opik.evaluation.metrics.score_result import ScoreResult

try:  # optional dependency
    import fasttext
except ImportError:  # pragma: no cover
    fasttext = None  # type: ignore


DetectorFn = Callable[[str], Tuple[str, float]]


class LanguageAdherenceMetric(BaseMetric):
    """
    Check whether text is written in the expected language.

    The metric relies on a fastText language identification model (or a
    user-supplied detector callable) to predict the language of the evaluated text
    and compares it with ``expected_language``. It outputs ``1.0`` when the detected
    language matches and ``0.0`` otherwise, along with the detected label and
    confidence score in ``metadata``.

    References:
      - fastText language identification models
        https://fasttext.cc/docs/en/language-identification.html
      - Joulin et al., "Bag of Tricks for Efficient Text Classification" (EACL 2017)
        https://aclanthology.org/E17-2068/

    Args:
        expected_language: Language code the text should conform to, e.g. ``"en"``.
        model_path: Path to a fastText language identification model. Required unless
            ``detector`` is provided.
        name: Display name for the metric result. Defaults to
            ``"language_adherence_metric"``.
        track: Whether to automatically track metric results. Defaults to ``True``.
        project_name: Optional tracking project name. Defaults to ``None``.
        detector: Optional callable accepting text and returning a
            ``(language, confidence)`` tuple. When provided, ``model_path`` is not
            needed.

    Example:
        >>> from opik.evaluation.metrics import LanguageAdherenceMetric
        >>> # Assuming `lid.176.ftz` is available locally for fastText
        >>> metric = LanguageAdherenceMetric(expected_language="en", model_path="lid.176.ftz")
        >>> result = metric.score("This response is written in English.")  # doctest: +SKIP
        >>> result.value  # doctest: +SKIP
        1.0
    """

    def __init__(
        self,
        expected_language: str,
        model_path: Optional[str] = None,
        name: str = "language_adherence_metric",
        track: bool = True,
        project_name: Optional[str] = None,
        detector: Optional[DetectorFn] = None,
    ) -> None:
        super().__init__(name=name, track=track, project_name=project_name)
        self._expected_language = expected_language
        self._detector_fn: DetectorFn
        self._model_path = model_path

        self._fasttext_model: Optional[Any]

        if detector is not None:
            self._detector_fn = detector
            self._fasttext_model = None
            return

        if fasttext is None:
            raise ImportError(
                "Install fasttext via `pip install fasttext` and provide a fastText language"
                " model (e.g., lid.176.ftz) or supply a custom detector callable."
            )
        if model_path is None:
            raise ValueError(
                "model_path is required when using the fastText-based detector."
            )
        self._fasttext_model = fasttext.load_model(model_path)
        self._detector_fn = self._predict_with_fasttext

    def score(self, output: str, **ignored_kwargs: Any) -> ScoreResult:
        processed = output
        if not processed.strip():
            raise MetricComputationError("Text is empty for language adherence check.")

        language, confidence = self._detector_fn(processed)
        adherence = 1.0 if language == self._expected_language else 0.0

        metadata = {
            "detected_language": language,
            "confidence": confidence,
            "expected_language": self._expected_language,
        }

        reason = (
            "Language adheres to expectation"
            if adherence == 1.0
            else f"Detected language '{language}' differs from expected '{self._expected_language}'"
        )

        return ScoreResult(
            value=adherence, name=self.name, reason=reason, metadata=metadata
        )

    def _predict_with_fasttext(self, text: str) -> tuple[str, float]:
        if self._fasttext_model is None:
            raise MetricComputationError(
                "fastText model is not loaded. Ensure that LanguageAdherenceMetric was initialized with a valid model_path and fastText is installed."
            )
        prediction = self._fasttext_model.predict(text)
        label = prediction[0][0] if prediction[0] else ""
        language = label.replace("__label__", "")
        confidence = float(prediction[1][0]) if prediction[1] else 0.0
        return language, confidence



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.dataset_expansion_response import DatasetExpansionResponse
from ..types.dataset_export_job_public import DatasetExportJobPublic
from ..types.dataset_item_changes_public import DatasetItemChangesPublic
from ..types.dataset_item_filter import DatasetItemFilter
from ..types.dataset_item_page_compare import DatasetItemPageCompare
from ..types.dataset_item_page_public import DatasetItemPagePublic
from ..types.dataset_item_public import DatasetItemPublic
from ..types.dataset_item_update import DatasetItemUpdate
from ..types.dataset_item_write import DatasetItemWrite
from ..types.dataset_item_write_source import DatasetItemWriteSource
from ..types.dataset_page_public import DatasetPagePublic
from ..types.dataset_public import DatasetPublic
from ..types.dataset_version_diff import DatasetVersionDiff
from ..types.dataset_version_page_public import DatasetVersionPagePublic
from ..types.dataset_version_public import DatasetVersionPublic
from ..types.json_node import JsonNode
from ..types.page_columns import PageColumns
from ..types.project_stats_public import ProjectStatsPublic
from ..types.span_enrichment_options import SpanEnrichmentOptions
from ..types.trace_enrichment_options import TraceEnrichmentOptions
from .raw_client import AsyncRawDatasetsClient, RawDatasetsClient
from .types.dataset_update_visibility import DatasetUpdateVisibility
from .types.dataset_write_visibility import DatasetWriteVisibility

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DatasetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDatasetsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDatasetsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDatasetsClient
        """
        return self._raw_client

    def apply_dataset_item_changes(
        self,
        id: str,
        *,
        request: DatasetItemChangesPublic,
        override: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetVersionPublic:
        """
        Apply delta changes (add, edit, delete) to a dataset version with conflict detection.

        This endpoint:
        - Creates a new version with the applied changes
        - Validates that baseVersion matches the latest version (unless override=true)
        - Returns 409 Conflict if baseVersion is stale and override is not set

        Use `override=true` query parameter to force version creation even with stale baseVersion.

        Parameters
        ----------
        id : str

        request : DatasetItemChangesPublic

        override : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionPublic
            Version created successfully

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.apply_dataset_item_changes(id='id', request={'key': 'value'
        }, )
        """
        _response = self._raw_client.apply_dataset_item_changes(
            id, request=request, override=override, request_options=request_options
        )
        return _response.data

    def batch_update_dataset_items(
        self,
        *,
        update: DatasetItemUpdate,
        ids: typing.Optional[typing.Sequence[str]] = OMIT,
        filters: typing.Optional[typing.Sequence[DatasetItemFilter]] = OMIT,
        dataset_id: typing.Optional[str] = OMIT,
        merge_tags: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update multiple dataset items

        Parameters
        ----------
        update : DatasetItemUpdate

        ids : typing.Optional[typing.Sequence[str]]
            List of dataset item IDs to update (max 1000). Mutually exclusive with 'filters'.

        filters : typing.Optional[typing.Sequence[DatasetItemFilter]]

        dataset_id : typing.Optional[str]
            Dataset ID. Required when using 'filters', optional when using 'ids'.

        merge_tags : typing.Optional[bool]
            If true, merge tags with existing tags instead of replacing them. Default: false. When using 'filters', this is automatically set to true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        from Opik import DatasetItemUpdate
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.batch_update_dataset_items(update=DatasetItemUpdate(), )
        """
        _response = self._raw_client.batch_update_dataset_items(
            update=update,
            ids=ids,
            filters=filters,
            dataset_id=dataset_id,
            merge_tags=merge_tags,
            request_options=request_options,
        )
        return _response.data

    def find_datasets(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        with_experiments_only: typing.Optional[bool] = None,
        with_optimizations_only: typing.Optional[bool] = None,
        prompt_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        sorting: typing.Optional[str] = None,
        filters: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetPagePublic:
        """
        Find datasets

        Parameters
        ----------
        page : typing.Optional[int]

        size : typing.Optional[int]

        with_experiments_only : typing.Optional[bool]

        with_optimizations_only : typing.Optional[bool]

        prompt_id : typing.Optional[str]

        name : typing.Optional[str]

        sorting : typing.Optional[str]

        filters : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetPagePublic
            Dataset resource

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.find_datasets()
        """
        _response = self._raw_client.find_datasets(
            page=page,
            size=size,
            with_experiments_only=with_experiments_only,
            with_optimizations_only=with_optimizations_only,
            prompt_id=prompt_id,
            name=name,
            sorting=sorting,
            filters=filters,
            request_options=request_options,
        )
        return _response.data

    def create_dataset(
        self,
        *,
        name: str,
        id: typing.Optional[str] = OMIT,
        visibility: typing.Optional[DatasetWriteVisibility] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create dataset

        Parameters
        ----------
        name : str

        id : typing.Optional[str]

        visibility : typing.Optional[DatasetWriteVisibility]

        tags : typing.Optional[typing.Sequence[str]]

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.create_dataset(name='name', )
        """
        _response = self._raw_client.create_dataset(
            name=name, id=id, visibility=visibility, tags=tags, description=description, request_options=request_options
        )
        return _response.data

    def create_or_update_dataset_items(
        self,
        *,
        items: typing.Sequence[DatasetItemWrite],
        dataset_name: typing.Optional[str] = OMIT,
        dataset_id: typing.Optional[str] = OMIT,
        batch_group_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create/update dataset items based on dataset item id

        Parameters
        ----------
        items : typing.Sequence[DatasetItemWrite]

        dataset_name : typing.Optional[str]
            If null, dataset_id must be provided

        dataset_id : typing.Optional[str]
            If null, dataset_name must be provided

        batch_group_id : typing.Optional[str]
            Optional batch group ID to group multiple batches into a single dataset version. If null, mutates the latest version instead of creating a new one.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        from Opik import DatasetItemWrite
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.create_or_update_dataset_items(items=[DatasetItemWrite(source="manual", data={'key': 'value'
        }, )], )
        """
        _response = self._raw_client.create_or_update_dataset_items(
            items=items,
            dataset_name=dataset_name,
            dataset_id=dataset_id,
            batch_group_id=batch_group_id,
            request_options=request_options,
        )
        return _response.data

    def create_dataset_items_from_csv(
        self,
        *,
        file: typing.Dict[str, typing.Optional[typing.Any]],
        dataset_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create dataset items from uploaded CSV file. CSV should have headers in the first row. Processing happens asynchronously in batches.

        Parameters
        ----------
        file : typing.Dict[str, typing.Optional[typing.Any]]

        dataset_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.create_dataset_items_from_csv(file={'key': 'value'
        }, dataset_id='dataset_id', )
        """
        _response = self._raw_client.create_dataset_items_from_csv(
            file=file, dataset_id=dataset_id, request_options=request_options
        )
        return _response.data

    def create_dataset_items_from_spans(
        self,
        dataset_id: str,
        *,
        span_ids: typing.Sequence[str],
        enrichment_options: SpanEnrichmentOptions,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create dataset items from spans with enriched metadata

        Parameters
        ----------
        dataset_id : str

        span_ids : typing.Sequence[str]
            Set of span IDs to add to the dataset

        enrichment_options : SpanEnrichmentOptions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        from Opik import SpanEnrichmentOptions
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.create_dataset_items_from_spans(dataset_id='dataset_id', span_ids=['span_ids'], enrichment_options=SpanEnrichmentOptions(), )
        """
        _response = self._raw_client.create_dataset_items_from_spans(
            dataset_id, span_ids=span_ids, enrichment_options=enrichment_options, request_options=request_options
        )
        return _response.data

    def create_dataset_items_from_traces(
        self,
        dataset_id: str,
        *,
        trace_ids: typing.Sequence[str],
        enrichment_options: TraceEnrichmentOptions,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create dataset items from traces with enriched metadata

        Parameters
        ----------
        dataset_id : str

        trace_ids : typing.Sequence[str]
            Set of trace IDs to add to the dataset

        enrichment_options : TraceEnrichmentOptions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        from Opik import TraceEnrichmentOptions
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.create_dataset_items_from_traces(dataset_id='dataset_id', trace_ids=['trace_ids'], enrichment_options=TraceEnrichmentOptions(), )
        """
        _response = self._raw_client.create_dataset_items_from_traces(
            dataset_id, trace_ids=trace_ids, enrichment_options=enrichment_options, request_options=request_options
        )
        return _response.data

    def get_dataset_by_id(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DatasetPublic:
        """
        Get dataset by id

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetPublic
            Dataset resource

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.get_dataset_by_id(id='id', )
        """
        _response = self._raw_client.get_dataset_by_id(id, request_options=request_options)
        return _response.data

    def update_dataset(
        self,
        id: str,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        visibility: typing.Optional[DatasetUpdateVisibility] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update dataset by id

        Parameters
        ----------
        id : str

        name : str

        description : typing.Optional[str]

        visibility : typing.Optional[DatasetUpdateVisibility]

        tags : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.update_dataset(id='id', name='name', )
        """
        _response = self._raw_client.update_dataset(
            id, name=name, description=description, visibility=visibility, tags=tags, request_options=request_options
        )
        return _response.data

    def delete_dataset(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete dataset by id

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.delete_dataset(id='id', )
        """
        _response = self._raw_client.delete_dataset(id, request_options=request_options)
        return _response.data

    def delete_dataset_by_name(
        self, *, dataset_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete dataset by name

        Parameters
        ----------
        dataset_name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.delete_dataset_by_name(dataset_name='dataset_name', )
        """
        _response = self._raw_client.delete_dataset_by_name(dataset_name=dataset_name, request_options=request_options)
        return _response.data

    def delete_dataset_items(
        self,
        *,
        item_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        dataset_id: typing.Optional[str] = OMIT,
        filters: typing.Optional[typing.Sequence[DatasetItemFilter]] = OMIT,
        batch_group_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete dataset items using one of two modes:
        1. **Delete by IDs**: Provide 'item_ids' to delete specific items by their IDs
        2. **Delete by filters**: Provide 'dataset_id' with optional 'filters' to delete items matching criteria

        When using filters, an empty 'filters' array will delete all items in the specified dataset.

        Parameters
        ----------
        item_ids : typing.Optional[typing.Sequence[str]]
            List of dataset item IDs to delete (max 1000). Use this to delete specific items by their IDs. Mutually exclusive with 'dataset_id' and 'filters'.

        dataset_id : typing.Optional[str]
            Dataset ID to scope the deletion. Required when using 'filters'. Mutually exclusive with 'item_ids'.

        filters : typing.Optional[typing.Sequence[DatasetItemFilter]]
            Filters to select dataset items to delete within the specified dataset. Must be used with 'dataset_id'. Mutually exclusive with 'item_ids'. Empty array means 'delete all items in the dataset'.

        batch_group_id : typing.Optional[str]
            Optional batch group ID to group multiple delete operations into a single dataset version. If null, mutates the latest version instead of creating a new one.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.delete_dataset_items()
        """
        _response = self._raw_client.delete_dataset_items(
            item_ids=item_ids,
            dataset_id=dataset_id,
            filters=filters,
            batch_group_id=batch_group_id,
            request_options=request_options,
        )
        return _response.data

    def delete_datasets_batch(
        self, *, ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete datasets batch

        Parameters
        ----------
        ids : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.delete_datasets_batch(ids=['ids'], )
        """
        _response = self._raw_client.delete_datasets_batch(ids=ids, request_options=request_options)
        return _response.data

    def download_dataset_export(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[bytes]:
        """
        Downloads the exported CSV file for a completed export job. This endpoint proxies the file download to avoid exposing internal storage URLs.

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            CSV file content

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.download_dataset_export(job_id='jobId', )
        """
        with self._raw_client.download_dataset_export(job_id, request_options=request_options) as r:
            yield from r.data

    def expand_dataset(
        self,
        id: str,
        *,
        model: str,
        sample_count: typing.Optional[int] = OMIT,
        preserve_fields: typing.Optional[typing.Sequence[str]] = OMIT,
        variation_instructions: typing.Optional[str] = OMIT,
        custom_prompt: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetExpansionResponse:
        """
        Generate synthetic dataset samples using LLM based on existing data patterns

        Parameters
        ----------
        id : str

        model : str
            The model to use for synthetic data generation

        sample_count : typing.Optional[int]
            Number of synthetic samples to generate

        preserve_fields : typing.Optional[typing.Sequence[str]]
            Fields to preserve patterns from original data

        variation_instructions : typing.Optional[str]
            Additional instructions for data variation

        custom_prompt : typing.Optional[str]
            Custom prompt to use for generation instead of auto-generated one

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetExpansionResponse
            Generated synthetic samples

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.expand_dataset(id='id', model='gpt-4', )
        """
        _response = self._raw_client.expand_dataset(
            id,
            model=model,
            sample_count=sample_count,
            preserve_fields=preserve_fields,
            variation_instructions=variation_instructions,
            custom_prompt=custom_prompt,
            request_options=request_options,
        )
        return _response.data

    def find_dataset_items_with_experiment_items(
        self,
        id: str,
        *,
        experiment_ids: str,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        filters: typing.Optional[str] = None,
        sorting: typing.Optional[str] = None,
        search: typing.Optional[str] = None,
        truncate: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetItemPageCompare:
        """
        Find dataset items with experiment items

        Parameters
        ----------
        id : str

        experiment_ids : str

        page : typing.Optional[int]

        size : typing.Optional[int]

        filters : typing.Optional[str]

        sorting : typing.Optional[str]

        search : typing.Optional[str]

        truncate : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetItemPageCompare
            Dataset item resource

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.find_dataset_items_with_experiment_items(id='id', experiment_ids='experiment_ids', )
        """
        _response = self._raw_client.find_dataset_items_with_experiment_items(
            id,
            experiment_ids=experiment_ids,
            page=page,
            size=size,
            filters=filters,
            sorting=sorting,
            search=search,
            truncate=truncate,
            request_options=request_options,
        )
        return _response.data

    def get_dataset_by_identifier(
        self, *, dataset_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetPublic:
        """
        Get dataset by name

        Parameters
        ----------
        dataset_name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetPublic
            Dataset resource

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.get_dataset_by_identifier(dataset_name='dataset_name', )
        """
        _response = self._raw_client.get_dataset_by_identifier(
            dataset_name=dataset_name, request_options=request_options
        )
        return _response.data

    def get_dataset_experiment_items_stats(
        self,
        id: str,
        *,
        experiment_ids: str,
        filters: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectStatsPublic:
        """
        Get experiment items stats for dataset

        Parameters
        ----------
        id : str

        experiment_ids : str

        filters : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectStatsPublic
            Experiment items stats resource

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.get_dataset_experiment_items_stats(id='id', experiment_ids='experiment_ids', )
        """
        _response = self._raw_client.get_dataset_experiment_items_stats(
            id, experiment_ids=experiment_ids, filters=filters, request_options=request_options
        )
        return _response.data

    def get_dataset_export_job(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetExportJobPublic:
        """
        Retrieves the current status of a dataset export job

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetExportJobPublic
            Export job details

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.get_dataset_export_job(job_id='jobId', )
        """
        _response = self._raw_client.get_dataset_export_job(job_id, request_options=request_options)
        return _response.data

    def get_dataset_export_jobs(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[DatasetExportJobPublic]:
        """
        Retrieves all export jobs for the workspace. This is used to restore the export panel state after page refresh.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[DatasetExportJobPublic]
            List of export jobs

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.get_dataset_export_jobs()
        """
        _response = self._raw_client.get_dataset_export_jobs(request_options=request_options)
        return _response.data

    def get_dataset_item_by_id(
        self, item_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetItemPublic:
        """
        Get dataset item by id

        Parameters
        ----------
        item_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetItemPublic
            Dataset item resource

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.get_dataset_item_by_id(item_id='itemId', )
        """
        _response = self._raw_client.get_dataset_item_by_id(item_id, request_options=request_options)
        return _response.data

    def patch_dataset_item(
        self,
        item_id: str,
        *,
        source: DatasetItemWriteSource,
        data: JsonNode,
        id: typing.Optional[str] = OMIT,
        trace_id: typing.Optional[str] = OMIT,
        span_id: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Partially update dataset item by id. Only provided fields will be updated.

        Parameters
        ----------
        item_id : str

        source : DatasetItemWriteSource

        data : JsonNode

        id : typing.Optional[str]

        trace_id : typing.Optional[str]

        span_id : typing.Optional[str]

        tags : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.patch_dataset_item(item_id='itemId', source="manual", data={'key': 'value'
        }, )
        """
        _response = self._raw_client.patch_dataset_item(
            item_id,
            source=source,
            data=data,
            id=id,
            trace_id=trace_id,
            span_id=span_id,
            tags=tags,
            request_options=request_options,
        )
        return _response.data

    def get_dataset_items(
        self,
        id: str,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        version: typing.Optional[str] = None,
        filters: typing.Optional[str] = None,
        truncate: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetItemPagePublic:
        """
        Get dataset items

        Parameters
        ----------
        id : str

        page : typing.Optional[int]

        size : typing.Optional[int]

        version : typing.Optional[str]

        filters : typing.Optional[str]

        truncate : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetItemPagePublic
            Dataset items resource

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.get_dataset_items(id='id', )
        """
        _response = self._raw_client.get_dataset_items(
            id,
            page=page,
            size=size,
            version=version,
            filters=filters,
            truncate=truncate,
            request_options=request_options,
        )
        return _response.data

    def get_dataset_items_output_columns(
        self,
        id: str,
        *,
        experiment_ids: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PageColumns:
        """
        Get dataset items output columns

        Parameters
        ----------
        id : str

        experiment_ids : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PageColumns
            Dataset item output columns

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.get_dataset_items_output_columns(id='id', )
        """
        _response = self._raw_client.get_dataset_items_output_columns(
            id, experiment_ids=experiment_ids, request_options=request_options
        )
        return _response.data

    def mark_dataset_export_job_viewed(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Marks a dataset export job as viewed by setting the viewed_at timestamp. This is used to track that a user has seen a failed job's error message. This operation is idempotent.

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.mark_dataset_export_job_viewed(job_id='jobId', )
        """
        _response = self._raw_client.mark_dataset_export_job_viewed(job_id, request_options=request_options)
        return _response.data

    def start_dataset_export(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetExportJobPublic:
        """
        Initiates an asynchronous CSV export job for the dataset. Returns immediately with job details for polling.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetExportJobPublic
            Existing export job in progress

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.start_dataset_export(id='id', )
        """
        _response = self._raw_client.start_dataset_export(id, request_options=request_options)
        return _response.data

    def stream_dataset_items(
        self,
        *,
        dataset_name: str,
        last_retrieved_id: typing.Optional[str] = OMIT,
        steam_limit: typing.Optional[int] = OMIT,
        dataset_version: typing.Optional[str] = OMIT,
        filters: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Stream dataset items

        Parameters
        ----------
        dataset_name : str

        last_retrieved_id : typing.Optional[str]

        steam_limit : typing.Optional[int]

        dataset_version : typing.Optional[str]

        filters : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Dataset items stream or error during process
        """
        with self._raw_client.stream_dataset_items(
            dataset_name=dataset_name,
            last_retrieved_id=last_retrieved_id,
            steam_limit=steam_limit,
            dataset_version=dataset_version,
            filters=filters,
            request_options=request_options,
        ) as r:
            yield from r.data

    def compare_dataset_versions(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetVersionDiff:
        """
        Compare the latest committed dataset version with the current draft state. This endpoint provides insights into changes made since the last version was committed. The comparison calculates additions, modifications, deletions, and unchanged items between the latest version snapshot and current draft.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionDiff
            Diff computed successfully

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.compare_dataset_versions(id='id', )
        """
        _response = self._raw_client.compare_dataset_versions(id, request_options=request_options)
        return _response.data

    def create_version_tag(
        self, id: str, version_hash: str, *, tag: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Add a tag to a specific dataset version for easy reference (e.g., 'baseline', 'v1.0', 'production')

        Parameters
        ----------
        id : str

        version_hash : str

        tag : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.create_version_tag(id='id', version_hash='versionHash', tag='tag', )
        """
        _response = self._raw_client.create_version_tag(id, version_hash, tag=tag, request_options=request_options)
        return _response.data

    def delete_version_tag(
        self, id: str, version_hash: str, tag: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove a tag from a dataset version. The version itself is not deleted, only the tag reference.

        Parameters
        ----------
        id : str

        version_hash : str

        tag : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.delete_version_tag(id='id', version_hash='versionHash', tag='tag', )
        """
        _response = self._raw_client.delete_version_tag(id, version_hash, tag, request_options=request_options)
        return _response.data

    def list_dataset_versions(
        self,
        id: str,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetVersionPagePublic:
        """
        Get paginated list of versions for a dataset, ordered by creation time (newest first)

        Parameters
        ----------
        id : str

        page : typing.Optional[int]

        size : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionPagePublic
            Dataset versions

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.list_dataset_versions(id='id', )
        """
        _response = self._raw_client.list_dataset_versions(id, page=page, size=size, request_options=request_options)
        return _response.data

    def restore_dataset_version(
        self, id: str, *, version_ref: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetVersionPublic:
        """
        Restores the dataset to a previous version state by creating a new version with items copied from the specified version. If the version is already the latest, returns it as-is (no-op).

        Parameters
        ----------
        id : str

        version_ref : str
            Version hash or tag to restore from

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionPublic
            Version restored successfully

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.restore_dataset_version(id='id', version_ref='version_ref', )
        """
        _response = self._raw_client.restore_dataset_version(
            id, version_ref=version_ref, request_options=request_options
        )
        return _response.data

    def retrieve_dataset_version(
        self, id: str, *, version_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetVersionPublic:
        """
        Get a specific version by its version name (e.g., 'v1', 'v373'). This is more efficient than paginating through all versions for large datasets.

        Parameters
        ----------
        id : str

        version_name : str
            Version name in format 'vN' (e.g., 'v1', 'v373')

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionPublic
            Dataset version

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.retrieve_dataset_version(id='id', version_name='v1', )
        """
        _response = self._raw_client.retrieve_dataset_version(
            id, version_name=version_name, request_options=request_options
        )
        return _response.data

    def update_dataset_version(
        self,
        id: str,
        version_hash: str,
        *,
        change_description: typing.Optional[str] = OMIT,
        tags_to_add: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetVersionPublic:
        """
        Update a dataset version's change_description and/or add new tags

        Parameters
        ----------
        id : str

        version_hash : str

        change_description : typing.Optional[str]
            Optional description of changes in this version

        tags_to_add : typing.Optional[typing.Sequence[str]]
            Optional list of tags to add to this version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionPublic
            Version updated successfully

        Examples
        --------
        from Opik import OpikApi
        client = OpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        client.datasets.update_dataset_version(id='id', version_hash='versionHash', )
        """
        _response = self._raw_client.update_dataset_version(
            id,
            version_hash,
            change_description=change_description,
            tags_to_add=tags_to_add,
            request_options=request_options,
        )
        return _response.data


class AsyncDatasetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDatasetsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDatasetsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDatasetsClient
        """
        return self._raw_client

    async def apply_dataset_item_changes(
        self,
        id: str,
        *,
        request: DatasetItemChangesPublic,
        override: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetVersionPublic:
        """
        Apply delta changes (add, edit, delete) to a dataset version with conflict detection.

        This endpoint:
        - Creates a new version with the applied changes
        - Validates that baseVersion matches the latest version (unless override=true)
        - Returns 409 Conflict if baseVersion is stale and override is not set

        Use `override=true` query parameter to force version creation even with stale baseVersion.

        Parameters
        ----------
        id : str

        request : DatasetItemChangesPublic

        override : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionPublic
            Version created successfully

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.apply_dataset_item_changes(id='id', request={'key': 'value'
            }, )
        asyncio.run(main())
        """
        _response = await self._raw_client.apply_dataset_item_changes(
            id, request=request, override=override, request_options=request_options
        )
        return _response.data

    async def batch_update_dataset_items(
        self,
        *,
        update: DatasetItemUpdate,
        ids: typing.Optional[typing.Sequence[str]] = OMIT,
        filters: typing.Optional[typing.Sequence[DatasetItemFilter]] = OMIT,
        dataset_id: typing.Optional[str] = OMIT,
        merge_tags: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update multiple dataset items

        Parameters
        ----------
        update : DatasetItemUpdate

        ids : typing.Optional[typing.Sequence[str]]
            List of dataset item IDs to update (max 1000). Mutually exclusive with 'filters'.

        filters : typing.Optional[typing.Sequence[DatasetItemFilter]]

        dataset_id : typing.Optional[str]
            Dataset ID. Required when using 'filters', optional when using 'ids'.

        merge_tags : typing.Optional[bool]
            If true, merge tags with existing tags instead of replacing them. Default: false. When using 'filters', this is automatically set to true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        from Opik import DatasetItemUpdate
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.batch_update_dataset_items(update=DatasetItemUpdate(), )
        asyncio.run(main())
        """
        _response = await self._raw_client.batch_update_dataset_items(
            update=update,
            ids=ids,
            filters=filters,
            dataset_id=dataset_id,
            merge_tags=merge_tags,
            request_options=request_options,
        )
        return _response.data

    async def find_datasets(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        with_experiments_only: typing.Optional[bool] = None,
        with_optimizations_only: typing.Optional[bool] = None,
        prompt_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        sorting: typing.Optional[str] = None,
        filters: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetPagePublic:
        """
        Find datasets

        Parameters
        ----------
        page : typing.Optional[int]

        size : typing.Optional[int]

        with_experiments_only : typing.Optional[bool]

        with_optimizations_only : typing.Optional[bool]

        prompt_id : typing.Optional[str]

        name : typing.Optional[str]

        sorting : typing.Optional[str]

        filters : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetPagePublic
            Dataset resource

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.find_datasets()
        asyncio.run(main())
        """
        _response = await self._raw_client.find_datasets(
            page=page,
            size=size,
            with_experiments_only=with_experiments_only,
            with_optimizations_only=with_optimizations_only,
            prompt_id=prompt_id,
            name=name,
            sorting=sorting,
            filters=filters,
            request_options=request_options,
        )
        return _response.data

    async def create_dataset(
        self,
        *,
        name: str,
        id: typing.Optional[str] = OMIT,
        visibility: typing.Optional[DatasetWriteVisibility] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create dataset

        Parameters
        ----------
        name : str

        id : typing.Optional[str]

        visibility : typing.Optional[DatasetWriteVisibility]

        tags : typing.Optional[typing.Sequence[str]]

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.create_dataset(name='name', )
        asyncio.run(main())
        """
        _response = await self._raw_client.create_dataset(
            name=name, id=id, visibility=visibility, tags=tags, description=description, request_options=request_options
        )
        return _response.data

    async def create_or_update_dataset_items(
        self,
        *,
        items: typing.Sequence[DatasetItemWrite],
        dataset_name: typing.Optional[str] = OMIT,
        dataset_id: typing.Optional[str] = OMIT,
        batch_group_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create/update dataset items based on dataset item id

        Parameters
        ----------
        items : typing.Sequence[DatasetItemWrite]

        dataset_name : typing.Optional[str]
            If null, dataset_id must be provided

        dataset_id : typing.Optional[str]
            If null, dataset_name must be provided

        batch_group_id : typing.Optional[str]
            Optional batch group ID to group multiple batches into a single dataset version. If null, mutates the latest version instead of creating a new one.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        from Opik import DatasetItemWrite
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.create_or_update_dataset_items(items=[DatasetItemWrite(source="manual", data={'key': 'value'
            }, )], )
        asyncio.run(main())
        """
        _response = await self._raw_client.create_or_update_dataset_items(
            items=items,
            dataset_name=dataset_name,
            dataset_id=dataset_id,
            batch_group_id=batch_group_id,
            request_options=request_options,
        )
        return _response.data

    async def create_dataset_items_from_csv(
        self,
        *,
        file: typing.Dict[str, typing.Optional[typing.Any]],
        dataset_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create dataset items from uploaded CSV file. CSV should have headers in the first row. Processing happens asynchronously in batches.

        Parameters
        ----------
        file : typing.Dict[str, typing.Optional[typing.Any]]

        dataset_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.create_dataset_items_from_csv(file={'key': 'value'
            }, dataset_id='dataset_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.create_dataset_items_from_csv(
            file=file, dataset_id=dataset_id, request_options=request_options
        )
        return _response.data

    async def create_dataset_items_from_spans(
        self,
        dataset_id: str,
        *,
        span_ids: typing.Sequence[str],
        enrichment_options: SpanEnrichmentOptions,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create dataset items from spans with enriched metadata

        Parameters
        ----------
        dataset_id : str

        span_ids : typing.Sequence[str]
            Set of span IDs to add to the dataset

        enrichment_options : SpanEnrichmentOptions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        from Opik import SpanEnrichmentOptions
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.create_dataset_items_from_spans(dataset_id='dataset_id', span_ids=['span_ids'], enrichment_options=SpanEnrichmentOptions(), )
        asyncio.run(main())
        """
        _response = await self._raw_client.create_dataset_items_from_spans(
            dataset_id, span_ids=span_ids, enrichment_options=enrichment_options, request_options=request_options
        )
        return _response.data

    async def create_dataset_items_from_traces(
        self,
        dataset_id: str,
        *,
        trace_ids: typing.Sequence[str],
        enrichment_options: TraceEnrichmentOptions,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create dataset items from traces with enriched metadata

        Parameters
        ----------
        dataset_id : str

        trace_ids : typing.Sequence[str]
            Set of trace IDs to add to the dataset

        enrichment_options : TraceEnrichmentOptions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        from Opik import TraceEnrichmentOptions
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.create_dataset_items_from_traces(dataset_id='dataset_id', trace_ids=['trace_ids'], enrichment_options=TraceEnrichmentOptions(), )
        asyncio.run(main())
        """
        _response = await self._raw_client.create_dataset_items_from_traces(
            dataset_id, trace_ids=trace_ids, enrichment_options=enrichment_options, request_options=request_options
        )
        return _response.data

    async def get_dataset_by_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetPublic:
        """
        Get dataset by id

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetPublic
            Dataset resource

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.get_dataset_by_id(id='id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_dataset_by_id(id, request_options=request_options)
        return _response.data

    async def update_dataset(
        self,
        id: str,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        visibility: typing.Optional[DatasetUpdateVisibility] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update dataset by id

        Parameters
        ----------
        id : str

        name : str

        description : typing.Optional[str]

        visibility : typing.Optional[DatasetUpdateVisibility]

        tags : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.update_dataset(id='id', name='name', )
        asyncio.run(main())
        """
        _response = await self._raw_client.update_dataset(
            id, name=name, description=description, visibility=visibility, tags=tags, request_options=request_options
        )
        return _response.data

    async def delete_dataset(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete dataset by id

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.delete_dataset(id='id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.delete_dataset(id, request_options=request_options)
        return _response.data

    async def delete_dataset_by_name(
        self, *, dataset_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete dataset by name

        Parameters
        ----------
        dataset_name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.delete_dataset_by_name(dataset_name='dataset_name', )
        asyncio.run(main())
        """
        _response = await self._raw_client.delete_dataset_by_name(
            dataset_name=dataset_name, request_options=request_options
        )
        return _response.data

    async def delete_dataset_items(
        self,
        *,
        item_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        dataset_id: typing.Optional[str] = OMIT,
        filters: typing.Optional[typing.Sequence[DatasetItemFilter]] = OMIT,
        batch_group_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete dataset items using one of two modes:
        1. **Delete by IDs**: Provide 'item_ids' to delete specific items by their IDs
        2. **Delete by filters**: Provide 'dataset_id' with optional 'filters' to delete items matching criteria

        When using filters, an empty 'filters' array will delete all items in the specified dataset.

        Parameters
        ----------
        item_ids : typing.Optional[typing.Sequence[str]]
            List of dataset item IDs to delete (max 1000). Use this to delete specific items by their IDs. Mutually exclusive with 'dataset_id' and 'filters'.

        dataset_id : typing.Optional[str]
            Dataset ID to scope the deletion. Required when using 'filters'. Mutually exclusive with 'item_ids'.

        filters : typing.Optional[typing.Sequence[DatasetItemFilter]]
            Filters to select dataset items to delete within the specified dataset. Must be used with 'dataset_id'. Mutually exclusive with 'item_ids'. Empty array means 'delete all items in the dataset'.

        batch_group_id : typing.Optional[str]
            Optional batch group ID to group multiple delete operations into a single dataset version. If null, mutates the latest version instead of creating a new one.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.delete_dataset_items()
        asyncio.run(main())
        """
        _response = await self._raw_client.delete_dataset_items(
            item_ids=item_ids,
            dataset_id=dataset_id,
            filters=filters,
            batch_group_id=batch_group_id,
            request_options=request_options,
        )
        return _response.data

    async def delete_datasets_batch(
        self, *, ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete datasets batch

        Parameters
        ----------
        ids : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.delete_datasets_batch(ids=['ids'], )
        asyncio.run(main())
        """
        _response = await self._raw_client.delete_datasets_batch(ids=ids, request_options=request_options)
        return _response.data

    async def download_dataset_export(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[bytes]:
        """
        Downloads the exported CSV file for a completed export job. This endpoint proxies the file download to avoid exposing internal storage URLs.

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            CSV file content

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.download_dataset_export(job_id='jobId', )
        asyncio.run(main())
        """
        async with self._raw_client.download_dataset_export(job_id, request_options=request_options) as r:
            async for data in r.data:
                yield data

    async def expand_dataset(
        self,
        id: str,
        *,
        model: str,
        sample_count: typing.Optional[int] = OMIT,
        preserve_fields: typing.Optional[typing.Sequence[str]] = OMIT,
        variation_instructions: typing.Optional[str] = OMIT,
        custom_prompt: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetExpansionResponse:
        """
        Generate synthetic dataset samples using LLM based on existing data patterns

        Parameters
        ----------
        id : str

        model : str
            The model to use for synthetic data generation

        sample_count : typing.Optional[int]
            Number of synthetic samples to generate

        preserve_fields : typing.Optional[typing.Sequence[str]]
            Fields to preserve patterns from original data

        variation_instructions : typing.Optional[str]
            Additional instructions for data variation

        custom_prompt : typing.Optional[str]
            Custom prompt to use for generation instead of auto-generated one

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetExpansionResponse
            Generated synthetic samples

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.expand_dataset(id='id', model='gpt-4', )
        asyncio.run(main())
        """
        _response = await self._raw_client.expand_dataset(
            id,
            model=model,
            sample_count=sample_count,
            preserve_fields=preserve_fields,
            variation_instructions=variation_instructions,
            custom_prompt=custom_prompt,
            request_options=request_options,
        )
        return _response.data

    async def find_dataset_items_with_experiment_items(
        self,
        id: str,
        *,
        experiment_ids: str,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        filters: typing.Optional[str] = None,
        sorting: typing.Optional[str] = None,
        search: typing.Optional[str] = None,
        truncate: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetItemPageCompare:
        """
        Find dataset items with experiment items

        Parameters
        ----------
        id : str

        experiment_ids : str

        page : typing.Optional[int]

        size : typing.Optional[int]

        filters : typing.Optional[str]

        sorting : typing.Optional[str]

        search : typing.Optional[str]

        truncate : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetItemPageCompare
            Dataset item resource

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.find_dataset_items_with_experiment_items(id='id', experiment_ids='experiment_ids', )
        asyncio.run(main())
        """
        _response = await self._raw_client.find_dataset_items_with_experiment_items(
            id,
            experiment_ids=experiment_ids,
            page=page,
            size=size,
            filters=filters,
            sorting=sorting,
            search=search,
            truncate=truncate,
            request_options=request_options,
        )
        return _response.data

    async def get_dataset_by_identifier(
        self, *, dataset_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetPublic:
        """
        Get dataset by name

        Parameters
        ----------
        dataset_name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetPublic
            Dataset resource

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.get_dataset_by_identifier(dataset_name='dataset_name', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_dataset_by_identifier(
            dataset_name=dataset_name, request_options=request_options
        )
        return _response.data

    async def get_dataset_experiment_items_stats(
        self,
        id: str,
        *,
        experiment_ids: str,
        filters: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectStatsPublic:
        """
        Get experiment items stats for dataset

        Parameters
        ----------
        id : str

        experiment_ids : str

        filters : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectStatsPublic
            Experiment items stats resource

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.get_dataset_experiment_items_stats(id='id', experiment_ids='experiment_ids', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_dataset_experiment_items_stats(
            id, experiment_ids=experiment_ids, filters=filters, request_options=request_options
        )
        return _response.data

    async def get_dataset_export_job(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetExportJobPublic:
        """
        Retrieves the current status of a dataset export job

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetExportJobPublic
            Export job details

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.get_dataset_export_job(job_id='jobId', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_dataset_export_job(job_id, request_options=request_options)
        return _response.data

    async def get_dataset_export_jobs(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[DatasetExportJobPublic]:
        """
        Retrieves all export jobs for the workspace. This is used to restore the export panel state after page refresh.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[DatasetExportJobPublic]
            List of export jobs

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.get_dataset_export_jobs()
        asyncio.run(main())
        """
        _response = await self._raw_client.get_dataset_export_jobs(request_options=request_options)
        return _response.data

    async def get_dataset_item_by_id(
        self, item_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetItemPublic:
        """
        Get dataset item by id

        Parameters
        ----------
        item_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetItemPublic
            Dataset item resource

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.get_dataset_item_by_id(item_id='itemId', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_dataset_item_by_id(item_id, request_options=request_options)
        return _response.data

    async def patch_dataset_item(
        self,
        item_id: str,
        *,
        source: DatasetItemWriteSource,
        data: JsonNode,
        id: typing.Optional[str] = OMIT,
        trace_id: typing.Optional[str] = OMIT,
        span_id: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Partially update dataset item by id. Only provided fields will be updated.

        Parameters
        ----------
        item_id : str

        source : DatasetItemWriteSource

        data : JsonNode

        id : typing.Optional[str]

        trace_id : typing.Optional[str]

        span_id : typing.Optional[str]

        tags : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.patch_dataset_item(item_id='itemId', source="manual", data={'key': 'value'
            }, )
        asyncio.run(main())
        """
        _response = await self._raw_client.patch_dataset_item(
            item_id,
            source=source,
            data=data,
            id=id,
            trace_id=trace_id,
            span_id=span_id,
            tags=tags,
            request_options=request_options,
        )
        return _response.data

    async def get_dataset_items(
        self,
        id: str,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        version: typing.Optional[str] = None,
        filters: typing.Optional[str] = None,
        truncate: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetItemPagePublic:
        """
        Get dataset items

        Parameters
        ----------
        id : str

        page : typing.Optional[int]

        size : typing.Optional[int]

        version : typing.Optional[str]

        filters : typing.Optional[str]

        truncate : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetItemPagePublic
            Dataset items resource

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.get_dataset_items(id='id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_dataset_items(
            id,
            page=page,
            size=size,
            version=version,
            filters=filters,
            truncate=truncate,
            request_options=request_options,
        )
        return _response.data

    async def get_dataset_items_output_columns(
        self,
        id: str,
        *,
        experiment_ids: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PageColumns:
        """
        Get dataset items output columns

        Parameters
        ----------
        id : str

        experiment_ids : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PageColumns
            Dataset item output columns

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.get_dataset_items_output_columns(id='id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_dataset_items_output_columns(
            id, experiment_ids=experiment_ids, request_options=request_options
        )
        return _response.data

    async def mark_dataset_export_job_viewed(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Marks a dataset export job as viewed by setting the viewed_at timestamp. This is used to track that a user has seen a failed job's error message. This operation is idempotent.

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.mark_dataset_export_job_viewed(job_id='jobId', )
        asyncio.run(main())
        """
        _response = await self._raw_client.mark_dataset_export_job_viewed(job_id, request_options=request_options)
        return _response.data

    async def start_dataset_export(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetExportJobPublic:
        """
        Initiates an asynchronous CSV export job for the dataset. Returns immediately with job details for polling.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetExportJobPublic
            Existing export job in progress

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.start_dataset_export(id='id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.start_dataset_export(id, request_options=request_options)
        return _response.data

    async def stream_dataset_items(
        self,
        *,
        dataset_name: str,
        last_retrieved_id: typing.Optional[str] = OMIT,
        steam_limit: typing.Optional[int] = OMIT,
        dataset_version: typing.Optional[str] = OMIT,
        filters: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Stream dataset items

        Parameters
        ----------
        dataset_name : str

        last_retrieved_id : typing.Optional[str]

        steam_limit : typing.Optional[int]

        dataset_version : typing.Optional[str]

        filters : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Dataset items stream or error during process
        """
        async with self._raw_client.stream_dataset_items(
            dataset_name=dataset_name,
            last_retrieved_id=last_retrieved_id,
            steam_limit=steam_limit,
            dataset_version=dataset_version,
            filters=filters,
            request_options=request_options,
        ) as r:
            async for data in r.data:
                yield data

    async def compare_dataset_versions(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetVersionDiff:
        """
        Compare the latest committed dataset version with the current draft state. This endpoint provides insights into changes made since the last version was committed. The comparison calculates additions, modifications, deletions, and unchanged items between the latest version snapshot and current draft.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionDiff
            Diff computed successfully

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.compare_dataset_versions(id='id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.compare_dataset_versions(id, request_options=request_options)
        return _response.data

    async def create_version_tag(
        self, id: str, version_hash: str, *, tag: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Add a tag to a specific dataset version for easy reference (e.g., 'baseline', 'v1.0', 'production')

        Parameters
        ----------
        id : str

        version_hash : str

        tag : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.create_version_tag(id='id', version_hash='versionHash', tag='tag', )
        asyncio.run(main())
        """
        _response = await self._raw_client.create_version_tag(
            id, version_hash, tag=tag, request_options=request_options
        )
        return _response.data

    async def delete_version_tag(
        self, id: str, version_hash: str, tag: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove a tag from a dataset version. The version itself is not deleted, only the tag reference.

        Parameters
        ----------
        id : str

        version_hash : str

        tag : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.delete_version_tag(id='id', version_hash='versionHash', tag='tag', )
        asyncio.run(main())
        """
        _response = await self._raw_client.delete_version_tag(id, version_hash, tag, request_options=request_options)
        return _response.data

    async def list_dataset_versions(
        self,
        id: str,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetVersionPagePublic:
        """
        Get paginated list of versions for a dataset, ordered by creation time (newest first)

        Parameters
        ----------
        id : str

        page : typing.Optional[int]

        size : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionPagePublic
            Dataset versions

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.list_dataset_versions(id='id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.list_dataset_versions(
            id, page=page, size=size, request_options=request_options
        )
        return _response.data

    async def restore_dataset_version(
        self, id: str, *, version_ref: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetVersionPublic:
        """
        Restores the dataset to a previous version state by creating a new version with items copied from the specified version. If the version is already the latest, returns it as-is (no-op).

        Parameters
        ----------
        id : str

        version_ref : str
            Version hash or tag to restore from

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionPublic
            Version restored successfully

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.restore_dataset_version(id='id', version_ref='version_ref', )
        asyncio.run(main())
        """
        _response = await self._raw_client.restore_dataset_version(
            id, version_ref=version_ref, request_options=request_options
        )
        return _response.data

    async def retrieve_dataset_version(
        self, id: str, *, version_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetVersionPublic:
        """
        Get a specific version by its version name (e.g., 'v1', 'v373'). This is more efficient than paginating through all versions for large datasets.

        Parameters
        ----------
        id : str

        version_name : str
            Version name in format 'vN' (e.g., 'v1', 'v373')

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionPublic
            Dataset version

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.retrieve_dataset_version(id='id', version_name='v1', )
        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve_dataset_version(
            id, version_name=version_name, request_options=request_options
        )
        return _response.data

    async def update_dataset_version(
        self,
        id: str,
        version_hash: str,
        *,
        change_description: typing.Optional[str] = OMIT,
        tags_to_add: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetVersionPublic:
        """
        Update a dataset version's change_description and/or add new tags

        Parameters
        ----------
        id : str

        version_hash : str

        change_description : typing.Optional[str]
            Optional description of changes in this version

        tags_to_add : typing.Optional[typing.Sequence[str]]
            Optional list of tags to add to this version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetVersionPublic
            Version updated successfully

        Examples
        --------
        from Opik import AsyncOpikApi
        import asyncio
        client = AsyncOpikApi(api_key="YOUR_API_KEY", workspace_name="YOUR_WORKSPACE_NAME", )
        async def main() -> None:
            await client.datasets.update_dataset_version(id='id', version_hash='versionHash', )
        asyncio.run(main())
        """
        _response = await self._raw_client.update_dataset_version(
            id,
            version_hash,
            change_description=change_description,
            tags_to_add=tags_to_add,
            request_options=request_options,
        )
        return _response.data



from __future__ import annotations

import json
import logging
from typing import Iterator, List, Optional, Set

from opik.rest_api import OpikApi
from opik.rest_api.types import (
    dataset_item as rest_dataset_item_read,
    dataset_version_public,
)
import opik.exceptions as exceptions
from opik.message_processing import streamer
from opik.rest_client_configurator import retry_decorator
from opik.api_objects import opik_query_language, rest_stream_parser
from . import dataset, dataset_item
from .. import experiment, constants
from ..experiment import experiments_client
from ...rest_api.core.api_error import ApiError

LOGGER = logging.getLogger(__name__)


def stream_dataset_items(
    rest_client: OpikApi,
    dataset_name: str,
    nb_samples: Optional[int] = None,
    batch_size: Optional[int] = None,
    dataset_item_ids: Optional[List[str]] = None,
    filter_string: Optional[str] = None,
    dataset_version: Optional[str] = None,
) -> Iterator[dataset_item.DatasetItem]:
    """
    Stream dataset items from the backend as a generator.

    Args:
        rest_client: The REST API client.
        dataset_name: Name of the dataset to stream items from.
        nb_samples: Maximum number of items to retrieve. If None, all items are streamed.
        batch_size: Maximum number of items to fetch per batch from the backend.
        dataset_item_ids: Optional list of specific item IDs to retrieve.
        filter_string: Optional OQL filter string to filter dataset items.
        dataset_version: Optional dataset version hash to filter items by a specific version.

    Yields:
        DatasetItem objects one at a time.
    """
    if batch_size is None:
        batch_size = constants.DATASET_STREAM_BATCH_SIZE

    last_retrieved_id: Optional[str] = None
    should_retrieve_more_items = True
    items_yielded = 0
    dataset_items_ids_left: Optional[Set[str]] = (
        set(dataset_item_ids) if dataset_item_ids else None
    )

    filters: Optional[str] = None
    if filter_string:
        oql = opik_query_language.OpikQueryLanguage.for_dataset_items(filter_string)
        filter_expressions = oql.get_filter_expressions()
        if filter_expressions:
            filters = json.dumps(filter_expressions)

    while should_retrieve_more_items:

        @retry_decorator.opik_rest_retry
        def _fetch_batch() -> List[rest_dataset_item_read.DatasetItem]:
            return rest_stream_parser.read_and_parse_stream(
                stream=rest_client.datasets.stream_dataset_items(
                    dataset_name=dataset_name,
                    last_retrieved_id=last_retrieved_id,
                    steam_limit=batch_size,
                    filters=filters,
                    dataset_version=dataset_version,
                ),
                item_class=rest_dataset_item_read.DatasetItem,
                nb_samples=nb_samples,
            )

        dataset_items = _fetch_batch()

        if len(dataset_items) == 0:
            should_retrieve_more_items = False
            break

        for item in dataset_items:
            item_id = item.id
            last_retrieved_id = item_id

            if dataset_items_ids_left is not None:
                if item_id not in dataset_items_ids_left:
                    continue
                else:
                    dataset_items_ids_left.remove(item_id)

            reconstructed_item = dataset_item.DatasetItem(
                id=item.id,
                trace_id=item.trace_id,
                span_id=item.span_id,
                source=item.source,
                **item.data,
            )

            yield reconstructed_item
            items_yielded += 1

            if nb_samples is not None and items_yielded >= nb_samples:
                should_retrieve_more_items = False
                break

            if dataset_items_ids_left is not None and len(dataset_items_ids_left) == 0:
                should_retrieve_more_items = False
                break

    if dataset_items_ids_left and len(dataset_items_ids_left) > 0:
        LOGGER.warning(
            "The following dataset items were not found in the dataset: %s",
            dataset_items_ids_left,
        )


def find_version_by_name(
    rest_client: OpikApi,
    dataset_id: str,
    version_name: str,
) -> Optional[dataset_version_public.DatasetVersionPublic]:
    """
    Find a dataset version by version name.

    Args:
        rest_client: The REST API client.
        dataset_id: The dataset ID to search versions in.
        version_name: Version name to search for (e.g., 'v1', 'v2').

    Returns:
        The DatasetVersionPublic if found, None otherwise.
    """
    try:
        return rest_client.datasets.retrieve_dataset_version(
            id=dataset_id, version_name=version_name
        )
    except ApiError as e:
        if e.status_code == 404:
            return None
        raise


def get_datasets(
    rest_client: OpikApi, max_results: int = 1000, sync_items: bool = True
) -> List[dataset.Dataset]:
    page_size = 100
    datasets: List[dataset.Dataset] = []

    page = 1
    while len(datasets) < max_results:
        page_datasets = rest_client.datasets.find_datasets(
            page=page,
            size=page_size,
        )

        if len(page_datasets.content) == 0:
            break

        for dataset_fern in page_datasets.content[: (max_results - len(datasets))]:
            dataset_ = dataset.Dataset(
                name=dataset_fern.name,
                description=dataset_fern.description,
                rest_client=rest_client,
                dataset_items_count=dataset_fern.dataset_items_count,
            )

            if sync_items:
                dataset_.__internal_api__sync_hashes__()

            datasets.append(dataset_)

        page += 1

    return datasets


def get_dataset_id(rest_client: OpikApi, dataset_name: str) -> str:
    try:
        dataset_id = rest_client.datasets.get_dataset_by_identifier(
            dataset_name=dataset_name
        ).id
    except ApiError as e:
        if e.status_code == 404:
            raise exceptions.DatasetNotFound(
                f"Dataset with the name {dataset_name} not found."
            ) from e
        raise

    return dataset_id


def get_dataset_experiments(
    rest_client: OpikApi,
    dataset_id: str,
    max_results: int,
    streamer: streamer.Streamer,
    experiments_client: experiments_client.ExperimentsClient,
) -> List[experiment.Experiment]:
    page_size = 100
    experiments: List[experiment.Experiment] = []

    page = 1
    while len(experiments) < max_results:
        page_experiments = rest_client.experiments.find_experiments(
            page=page,
            size=page_size,
            dataset_id=dataset_id,
        )

        if len(page_experiments.content) == 0:
            break

        for experiment_ in page_experiments.content[: max_results - len(experiments)]:
            experiments.append(
                experiment.Experiment(
                    id=experiment_.id,
                    name=experiment_.name,
                    dataset_name=experiment_.dataset_name,
                    rest_client=rest_client,
                    streamer=streamer,
                    experiments_client=experiments_client,
                    tags=experiment_.tags,
                )
            )

        page += 1

    return experiments
